
#include "BBnettraffic.h"

PLUGIN_STRUCT *plugin;

struct plugindata {
   long       timer;
   bool       reconfUpdate;
} plugindata;

int adapter;
UINT timer=0;
DATATYPE bytes[2]={0,0},average[2]={0,0};
double price,cost;

LabelList labellist;
Graph *graph[2];


const char * BBpluginCallback_InfoQuery (int field)
{
	switch (field) {
	       case PLUGIN_NAME:      return(szAppName);
          case PLUGIN_CLASSNAME: return(szClassName);
          case PLUGIN_VERSION:   return(szInfoVersion);  
          case PLUGIN_AUTHOR:    return(szInfoAuthor);   
          case PLUGIN_RELEASE:   return(szInfoRelDate); 
          case PLUGIN_LINK:      return(szInfoLink);   
          case PLUGIN_EMAIL:     return(szInfoEmail);   
          case PLUGIN_NAMEVER:   return(szVersion);
	}
   return(NULL);
}

bool BBpluginCallback_OnLoadPlugin()
{ 
   plugin=BBpluginExport_GetPlugin();

   if (!InitInterface()) {
      MessageBox(NULL,"Unable to load symbol \"GetIfTable\" from \"iphlpapi.dll\"!","Fatal",MB_OK);
      return(false);
   }

   return(true);
}

void BBpluginCallback_OnUnloadPlugin()
{
   KillTimer(NULL,timer);
   labellist.clear();
   delete graph[GIN];
   delete graph[GOUT];
}

bool BBpluginCallback_OnMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   return(false);
}

void BBpluginCallback_OnBroam (const char * broam)
{
   if (BBplugin_IsBroam(&broam,ADAPTER_COMMAND)) {
      adapter=*broam-'0';
      reset();
   }
}

#ifdef GDIPLUS_IN_USE
void BBpluginCallback_OnPaint (HDC hdc)
{
   Gdiplus::Graphics g(hdc);

   Gdiplus::Font font(hdc,BBpluginExport_GetBB()->font);
   COLORREF c=plugin->windowStyleItem?plugin->windowStyleItem->TextColor:RGB(0,0,0);
   Gdiplus::SolidBrush brush(Gdiplus::Color(255,GetRValue(c),GetGValue(c),GetBValue(c)));

   for (int gr=GIN;gr<=GOUT;gr++) graph[gr]->paint(g,hdc,font);

   labellist.paintLabels(g,font,brush);
}
#else
void BBpluginCallback_OnPaint (HDC hdc)
{
   for (int gr=GIN;gr<=GOUT;gr++) graph[gr]->paint(hdc);

   labellist.paintLabels(hdc);
}
#endif

bool BBpluginCallback_OnCreateMenu (Menu * menu)
{
   Menu *adapterMenu;
   const char *broam_prefix=BBpluginExport_GetBroamPrefix();

   adapterMenu=MakeNamedMenu("Adapter",szAppName"_IDAdapter",true);
   for (int i=0;i<GetNumberOfAdapters();i++) {
       char desc[256];
       GetAdapterDescription(i,desc);
       char command[50];
       sprintf(command,"%s%s%d",broam_prefix,ADAPTER_COMMAND,i);
       MakeMenuItem(adapterMenu,desc,command,i==adapter);
   }
   MakeSubmenu(menu,adapterMenu,"Adapter");
   
   return(true);
}

void BBpluginCallback_OnConfigure (void)
{
   labellist.clear();

   plugindata.timer=BBplugin_ReadInt("timer",1000);
   plugindata.reconfUpdate=BBplugin_ReadBool("reconfigureUpdate",false);
 
   labellist.addLabel(new Label("receiveLabel"));
   labellist.addLabel(new Label("sendLabel"));
   labellist.addLabel(new Label("bothLabel"));
   labellist.addLabel(new Label("costLabel"));

   for (int g=GIN;g<=GOUT;g++) {
       if (graph[g]) delete graph[g];
       graph[g]=new Graph(g==GIN?"receiveGraph":"sendGraph");
       //graph[g]->setFixedMaxValue(5);
   }

   price=atof(BBplugin_ReadString("price","1"));
   adapter=BBplugin_ReadInt("adapter",0);

   reset();
   if (timer) KillTimer(NULL,timer);
   timer=SetTimer(NULL,NULL,plugindata.timer,(TIMERPROC)TimerProc);

   if (plugindata.reconfUpdate) TimerProc();
}

void BBpluginCallback_OnSaveConfiguration (void)
{
   BBplugin_WriteInt("adapter",adapter);
}


TIMERPROC TimerProc (void)
{
   for (int g=GIN;g<=GOUT;g++) {
       DATATYPE now=GetTrasmittedBytes(adapter,(g==GIN)?in:out);
       average[g]=(DATATYPE)(((double)(now-bytes[g]))/(double)(plugindata.timer/(double)1000));
       bytes[g]=now;

       graph[g]->addValue(average[g]);
       char *text=graph[g]->getDisplayTextPtr();
       valueOut(text,average[g]);
       strcpy(text+strlen(text),"/s");
   }

   cost=((double)(bytes[LIN]+bytes[LOUT]))/_2NA10*price;

   valueOut(labellist.getLabel(LIN)->getContentPtr(),bytes[LIN]);
   valueOut(labellist.getLabel(LOUT)->getContentPtr(),bytes[LOUT]);
   valueOut(labellist.getLabel(LIO)->getContentPtr(),bytes[LIN]+bytes[LOUT]);
   sprintf(labellist.getLabel(LCOST)->getContentPtr(),"%.2f",cost);

   BBpluginExport_Repaint();
   return(0);
}

void valueOut (char * text, DATATYPE value)
{
   if (value<_2NA10) sprintf(text,"%I64d B",value);
   if (value>=_2NA10 && value<_2NA20) sprintf(text,"%.3f KB",(double)value/_2NA10);
   if (value>=_2NA20 && value<_2NA30) sprintf(text,"%.1f MB",(double)value/_2NA20);
   if (value>=_2NA30) sprintf(text,"%.2f GB",(double)value/_2NA30);
}

void reset (void)
{
   bytes[GIN]=GetTrasmittedBytes(adapter,in);
   bytes[GOUT]=GetTrasmittedBytes(adapter,out);
   for (int g=GIN;g<=GOUT;g++) graph[g]->reset();
}

