
#include "BBping.h"

PLUGIN_STRUCT *plugin;

struct plugindata {
   int      timer;
   bool     timerUpdate,reconfUpdate;
   int      datasize;
   int      maxWaitTime;
} plugindata;

UINT timer;
bool thread_running=false;

LabelList comps;


const char * BBpluginCallback_InfoQuery (int field)
{
	switch (field) {
          case PLUGIN_NAME:      return(szAppName);
          case PLUGIN_CLASSNAME: return(szClassName);
          case PLUGIN_VERSION:   return(szInfoVersion);  
          case PLUGIN_AUTHOR:    return(szInfoAuthor);   
          case PLUGIN_RELEASE:   return(szInfoRelDate); 
          case PLUGIN_LINK:      return(szInfoLink);   
          case PLUGIN_EMAIL:     return(szInfoEmail);   
          case PLUGIN_NAMEVER:   return(szVersion);
   }
   return(NULL);
}

bool BBpluginCallback_OnLoadPlugin()
{ 
   plugin=BBpluginExport_GetPlugin();

   return(true);
}

void BBpluginCallback_OnUnloadPlugin()
{
   KillTimer(NULL,timer);
}

bool BBpluginCallback_OnMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   return(false);
}

void BBpluginCallback_OnBroam (const char * broam)
{
   if (BBplugin_IsBroam(&broam,TIMERUPDATE_COMMAND)) {
      plugindata.timerUpdate=!plugindata.timerUpdate;
   }
   if (BBplugin_IsBroam(&broam,FORCEUPDATE_COMMAND)) {
      LaunchUpdate();
   }
}

void BBpluginCallback_OnPaint (HDC hdc)
{
   comps.paintLabels(hdc);
}

bool BBpluginCallback_OnCreateMenu (Menu * mainMenu)
{
   char broam[1000];
   const char *broam_prefix=BBpluginExport_GetBroamPrefix();

   sprintf(broam,"%s%s",broam_prefix,TIMERUPDATE_COMMAND);
   MakeMenuItem(mainMenu,"Timer Update",broam,plugindata.timerUpdate);
   sprintf(broam,"%s%s",broam_prefix,FORCEUPDATE_COMMAND);
   MakeMenuItem(mainMenu,"Force Update!",broam,false);

   return(true);
}

void BBpluginCallback_OnConfigure (void)
{
   comps.clear();

   plugindata.timer=BBplugin_ReadInt("timer",1);
   plugindata.timerUpdate=BBplugin_ReadBool("timerUpdate",false);
   plugindata.reconfUpdate=BBplugin_ReadBool("reconfigureUpdate",false);
   plugindata.datasize=BBplugin_ReadInt("dataSize",32);
   plugindata.maxWaitTime=BBplugin_ReadInt("maxWaitTime",1000);

   int cmps=BBplugin_ReadInt("computerCount",0);
   for (int i=0;i<cmps;i++) {
       char txt[1024];
       sprintf(txt,"computer[%d]",i+1);
       comps.addLabel(new Label(txt));
   }

   if (timer) KillTimer(NULL,timer);
   timer=SetTimer(NULL,NULL,plugindata.timer*1000*60,(TIMERPROC)TimerProc);

   if (plugindata.reconfUpdate) LaunchUpdate();
}

void BBpluginCallback_OnSaveConfiguration (void)
{
   BBplugin_WriteBool("timerUpdate",plugindata.timerUpdate);
}


TIMERPROC TimerProc (void)
{
   if (plugindata.timerUpdate) LaunchUpdate();
   return(0);
}

void LaunchUpdate (void)
{
   if (!thread_running) {
      DWORD thandle;
      CreateThread(NULL,0,PingThread,0,0,&thandle);
   }
}
   
DWORD WINAPI PingThread (LPVOID)
{
   thread_running=true;

   void *data=malloc(plugindata.datasize);
   DWORD replyBufferSize=sizeof(ICMP_ECHO_REPLY)+plugindata.datasize;
   void *replyBuffer=malloc(replyBufferSize);

   HANDLE h=IcmpCreateFile();

   for (int c=0;c<comps.getNumLabels();c++) {
       Label *label=comps.getLabel(c);
       IPAddr ip;//=inet_addr(label->getID());

       if ((ip=inet_addr(label->getID()))==INADDR_NONE) {
          hostent *host;
          if ((host=gethostbyname(label->getID()))!=NULL) {
             memcpy(&ip,host->h_addr,host->h_length);
          }
          else {
//             MessageBox(plugin,"Unable to resolve hostname!",label->getID(),MB_OK|MB_ICONERROR|MB_TOPMOST);
             continue;
          }
       }

       int received=IcmpSendEcho(h,ip,data,plugindata.datasize,NULL,replyBuffer,replyBufferSize,plugindata.maxWaitTime);
         
       if (received) {
          PICMP_ECHO_REPLY pEchoReply=(PICMP_ECHO_REPLY)replyBuffer;
          sprintf(label->getContentPtr(),"%d ms",pEchoReply->RoundTripTime);
       }
       else sprintf(label->getContentPtr(),"n/a");

       BBpluginExport_Repaint();
   }

   IcmpCloseHandle(h);
   free(data);
   free(replyBuffer);

   thread_running=false;
   return(0);
}


