
#include "BBpluginSDK.h"

struct {
   const char 
      *pAppName, *pClassName, *pInfoVersion, *pVersion,
      *pInfoAuthor, *pInfoRelDate, *pInfoLink, *pInfoEmail;
} pinfo;

BB_STRUCT BB;
PLUGIN_STRUCT plugin;

SETLAYEREDWINDOW SetLayeredWindow;
HBRUSH sctBackground;

struct {
   char broam_prefix[100],
        menu_readsettings[100],menu_writesettings[100],menu_editsettings[100],
        menu_about[100],
        window_show[100],window_hide[100],window_toggle[100];
} broams;


// Returns pointer to the "BB" structure ------------------------------------
BB_STRUCT * BBpluginExport_GetBB (void)
{
   return(&BB);
}
// Returns pointer to the "plugin" structure --------------------------------
PLUGIN_STRUCT * BBpluginExport_GetPlugin (void)
{
   return(&plugin);
}


/*// Forces the plugin to reload configuration
void BBpluginExport_Configure (void)
{
   BBplugin_Configure();
   BBplugin_UpdateWindowPrefs(plugin.hwnd);
}*/

// Returns broam prefix for this plugin
const char * BBpluginExport_GetBroamPrefix (void)
{
   return(broams.broam_prefix);
}

// Returns single color transparency brush
HBRUSH * BBpluginExport_GetSctBrush (void)
{
   return(&sctBackground);
}

// Sends bro@m
void BBpluginExport_SendBroam (const char * broam)
{
   SendMessage(BB.hwnd,BB_BROADCAST,0,(LPARAM)broam);
}

// Forces repainting of the window
void BBpluginExport_Repaint (void)
{
   BBplugin_InvalidateWindow();
}

// Asks higher level for plugin information ---------------------------------
void BBplugin_FetchInfo (void)
{
   pinfo.pAppName=BBpluginCallback_InfoQuery(PLUGIN_NAME);
   pinfo.pClassName=BBpluginCallback_InfoQuery(PLUGIN_CLASSNAME);
   pinfo.pInfoVersion=BBpluginCallback_InfoQuery(PLUGIN_VERSION);
   pinfo.pInfoAuthor=BBpluginCallback_InfoQuery(PLUGIN_AUTHOR);
   pinfo.pInfoRelDate=BBpluginCallback_InfoQuery(PLUGIN_RELEASE);
   pinfo.pInfoLink=BBpluginCallback_InfoQuery(PLUGIN_LINK);
   pinfo.pInfoEmail=BBpluginCallback_InfoQuery(PLUGIN_EMAIL);
   pinfo.pVersion=BBpluginCallback_InfoQuery(PLUGIN_NAMEVER);

   sprintf(broams.broam_prefix,"@%s.",pinfo.pAppName);
   sprintf(broams.menu_readsettings,"%ssettings.read",broams.broam_prefix);
   sprintf(broams.menu_writesettings,"%ssettings.write",broams.broam_prefix);
   sprintf(broams.menu_editsettings,"%ssettings.edit",broams.broam_prefix);
   sprintf(broams.menu_about,"%sabout",broams.broam_prefix);
   sprintf(broams.window_show,"%sshow",broams.broam_prefix);
   sprintf(broams.window_hide,"%shide",broams.broam_prefix);
   sprintf(broams.window_toggle,"%stoggle",broams.broam_prefix);
}

bool BBplugin_LoadPlugin (HINSTANCE hPluginInstance)
{
   BB.hwnd=GetBBWnd();
   plugin.hInstance=hPluginInstance;

   BBplugin_FetchInfo();

   GetModuleFileName(plugin.hInstance,plugin.directory,sizeof(plugin.directory));
   strcpy(plugin.rcfile,plugin.directory);
   *(strrchr(plugin.directory,'\\')+1)='\0';
   strcpy(strrchr(plugin.rcfile,'.')+1,"rc");

   BBplugin_Configure();

   SetLayeredWindow=(SETLAYEREDWINDOW)GetProcAddress(LoadLibrary(TEXT("user32.dll")),"SetLayeredWindowAttributes");

   WNDCLASS wc;
   ZeroMemory(&wc,sizeof(wc));
   wc.lpfnWndProc=BBplugin_WindowProc;      
   wc.hInstance=plugin.hInstance;    
   wc.lpszClassName=pinfo.pClassName;    
   wc.hCursor=LoadCursor(NULL,IDC_ARROW);
   wc.style=CS_DBLCLKS;

	if (!RegisterClass(&wc)) {
		MessageBox(BB.hwnd,"Error registering window class","Error",
         MB_OK|MB_ICONERROR|MB_TOPMOST);
		return(false);
	}

   plugin.hwnd=CreateWindowEx(
      WS_EX_TOOLWINDOW | (BB.slit?0:WS_EX_LAYERED),
      pinfo.pClassName,
      NULL,
      WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE,
      0,0,
      0,0,
      NULL,
      NULL,
      plugin.hInstance,
      NULL);
   if (plugin.hwnd==NULL) {
      MessageBox(BB.hwnd,"Error creating plugin window","Error",
         MB_OK|MB_ICONERROR|MB_TOPMOST);
      UnregisterClass(pinfo.pClassName,plugin.hInstance);
      return(false);
   }

   return(true);
}

static int msgs[]={BB_RECONFIGURE,BB_BROADCAST,0};

LRESULT CALLBACK BBplugin_WindowProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   if (BBpluginCallback_OnMessage(hwnd,message,wParam,lParam)) return(0);

   switch (message) {
          case WM_CREATE:
               SendMessage(BB.hwnd,BB_REGISTERMESSAGE,(WPARAM)hwnd,(LPARAM)msgs);
               MakeSticky(hwnd);
               BBplugin_UpdateWindowPrefs(hwnd);   // predavame hwnd, protoze v tuhle chvili jeste plugin.hwnd nemusi mit hodnotu
               break;
          case WM_PAINT: 
               BBplugin_OnPaint(hwnd);
               break;
          
          case WM_LBUTTONDOWN:
               SetFocus(hwnd);         // kvuli ztraceni focusu ve slitu
               if (GetAsyncKeyState(VK_CONTROL) & 0x8000) {
                  PostMessage(hwnd,WM_SYSCOMMAND,0xF012,0);          // start moving
                  return(0);
               }
               if (GetAsyncKeyState(VK_MENU) & 0x8000) {
  				      PostMessage(hwnd,WM_SYSCOMMAND,0xF008,0);          // start sizing
                  return(0);
               }
               BBplugin_InvalidateWindow();
               break;
          case WM_RBUTTONUP:
               BBplugin_CreateMenu();
               break;

          case WM_WINDOWPOSCHANGED: {
               WINDOWPOS *wp=(WINDOWPOS*)lParam;
               if (!BB.slit) {
                  plugin.x=wp->x;
                  plugin.y=wp->y;
               }
               if (!(wp->flags & SWP_NOSIZE)) {
                  plugin.width=wp->cx;
                  plugin.height=wp->cy;
                  BBplugin_InvalidateWindow();
               }
               break;
          }
          case WM_EXITSIZEMOVE:
               if (BB.slit) SendMessage(BB.slit,SLIT_UPDATE,0,(LPARAM)plugin.hwnd);
               break;
          case WM_CLOSE:
               break;

          case BB_RECONFIGURE:
               BBplugin_Configure();
               BBplugin_UpdateWindowPrefs(hwnd);
               break;
          case BB_BROADCAST: {
               const char *msg=(LPCSTR)lParam;
               if (BBplugin_IsBroam(&msg,"@BBShowPlugins") && plugin.pluginToggle) ShowWindow(hwnd,SW_SHOW);
               if (BBplugin_IsBroam(&msg,"@BBHidePlugins") && plugin.pluginToggle) ShowWindow(hwnd,SW_HIDE);
               
               if (BBplugin_IsBroam(&msg,broams.window_show)) {
                  ShowWindow(plugin.hwnd,SW_SHOW);
               }
               if (BBplugin_IsBroam(&msg,broams.window_hide)) {
                  ShowWindow(plugin.hwnd,SW_HIDE);
               }
               if (BBplugin_IsBroam(&msg,broams.window_toggle)) {
                  ShowWindow(plugin.hwnd,IsWindowVisible(plugin.hwnd)?SW_HIDE:SW_SHOW);
               }

               if (BBplugin_IsBroam(&msg,broams.menu_readsettings)) {
                  BBplugin_Configure();
                  BBplugin_UpdateWindowPrefs(plugin.hwnd);
               }
               if (BBplugin_IsBroam(&msg,broams.menu_writesettings)) 
                  BBplugin_SaveConfiguration();
               if (BBplugin_IsBroam(&msg,broams.menu_editsettings)) 
                  BBplugin_Execute(plugin.rcfile);
               if (BBplugin_IsBroam(&msg,broams.menu_about)) 
                  BBplugin_AboutBox();

               msg+=strlen(broams.broam_prefix);
               BBpluginCallback_OnBroam(msg);
               break;
          }

          default:
               return(DefWindowProc(hwnd,message,wParam,lParam));
   }
   return(0);
}

void BBplugin_OnPaint (HWND hwnd)
{
   PAINTSTRUCT ps;
	HDC hdc=BeginPaint(hwnd,&ps);  

   HDC myDC=CreateCompatibleDC(hdc);
	HGDIOBJ previousBmp;

   if (plugin.layout==NULL) {
      plugin.layout=CreateCompatibleBitmap(hdc,plugin.width,plugin.height);
      previousBmp=SelectObject(myDC,plugin.layout);

      RECT rect;
      SetRect(&rect,0,0,plugin.width,plugin.height);
      if (!plugin.sctEnabled) 
         MakeStyleGradient(myDC,&rect,plugin.windowStyleItem,true);
      else 
         FillRect(myDC,&rect,sctBackground);

      HGDIOBJ oldfont=SelectObject(myDC,BB.font);
      SetBkMode(myDC,TRANSPARENT);
      SetTextColor(myDC,plugin.windowStyleItem->TextColor);
      
      BBpluginCallback_OnPaint(myDC);

      SelectObject(myDC,oldfont);
   }
   else previousBmp=SelectObject(myDC,plugin.layout);

	BitBlt(hdc,
	 		 ps.rcPaint.left,ps.rcPaint.top,ps.rcPaint.right-ps.rcPaint.left,ps.rcPaint.bottom-ps.rcPaint.top,
			 myDC,
			 ps.rcPaint.left,ps.rcPaint.top,
			 SRCCOPY);

   SelectObject(myDC,previousBmp);
	DeleteDC(myDC);

   EndPaint(hwnd,&ps);
}

void BBplugin_InvalidateWindow (void) 
{
   if (plugin.layout) {
      DeleteObject(plugin.layout);
      plugin.layout=NULL;
   }
   InvalidateRect(plugin.hwnd,NULL,false);
}

void BBplugin_CreateMenu (void)
{
   Menu *mainMenu,*settingsMenu;
   char id[1000];

   sprintf(id,"%s_IDMain",pinfo.pAppName);
   mainMenu=MakeNamedMenu(pinfo.pAppName,id,true);

   if (BBpluginCallback_OnCreateMenu(mainMenu))
      MakeMenuNOP(mainMenu,NULL);

   sprintf(id,"%s_IDSettings",pinfo.pAppName);
   settingsMenu=MakeNamedMenu("Settings",id,true);
   MakeMenuItem(settingsMenu,"read settings",broams.menu_readsettings,false);
   MakeMenuItem(settingsMenu,"write settings",broams.menu_writesettings,false);
   MakeMenuItem(settingsMenu,"edit settings",broams.menu_editsettings,false);
   MakeSubmenu(mainMenu,settingsMenu,"Settings");
   MakeMenuItem(mainMenu,"About",broams.menu_about,false);   
   
   ShowMenu(mainMenu);
}

int BBplugin_ReadInt (const char * field, int def)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   return(ReadInt(plugin.rcfile,expr,def));
}
bool BBplugin_ReadBool (const char * field, bool def)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   return(ReadBool(plugin.rcfile,expr,def));
}
COLORREF BBplugin_ReadColor (const char * field, const char * def)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   return(ReadColor(plugin.rcfile,expr,def));
}
const char * BBplugin_ReadString (const char * field, const char * def)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   return(ReadString(plugin.rcfile,expr,def));
}
void BBplugin_WriteInt (const char * field, int value)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   WriteInt(plugin.rcfile,expr,value);
}
void BBplugin_WriteBool (const char * field, bool value)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   WriteBool(plugin.rcfile,expr,value);
}
void BBplugin_WriteString (const char * field, const char * value)
{
   char expr[1000];
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
   WriteString(plugin.rcfile,expr,value);
}

void BBplugin_SaveConfiguration (void)
{
   BBplugin_WriteInt("x",plugin.x);
   BBplugin_WriteInt("y",plugin.y);
   BBplugin_WriteInt("width",plugin.width);
   BBplugin_WriteInt("height",plugin.height);

   BBpluginCallback_OnSaveConfiguration();
}

HICON BBplugin_LoadIcon (LPCSTR path)
{
   HICON icon=NULL;
   if (path!=NULL) {
      char dirpath[MAX_PATH];
      sprintf(dirpath,"%s%s",plugin.directory,path);
      icon=(HICON)LoadImage(0,dirpath,IMAGE_ICON,0,0,LR_LOADFROMFILE);
      if (icon==NULL) 
         MessageBox(BB.hwnd,dirpath,"Cannot load icon",MB_OK|MB_ICONERROR|MB_TOPMOST);
   }
   return(icon);
}

int BBplugin_ParseStyle (const char * styletxt)
{
   int style=SN_TOOLBAR;
   if (!stricmp(styletxt,"button")) style=SN_TOOLBARBUTTON;
   if (!stricmp(styletxt,"buttonpressed")) style=SN_TOOLBARBUTTONP;
   if (!stricmp(styletxt,"label")) style=SN_TOOLBARLABEL;
   if (!stricmp(styletxt,"windowlabel")) style=SN_TOOLBARWINDOWLABEL;
   if (!stricmp(styletxt,"clock")) style=SN_TOOLBARCLOCK;
   return(style);
}

void BBplugin_UpdateWindowPrefs (HWND hwnd)
{
   SetWindowPos(hwnd,plugin.alwaysOnTop?HWND_TOPMOST:HWND_NOTOPMOST,
                plugin.x,plugin.y,plugin.width,plugin.height,SWP_NOACTIVATE|SWP_NOSENDCHANGING);
   if (SetLayeredWindow) {
      SetLayeredWindow(hwnd,0,(BYTE)(plugin.alphaEnabled?plugin.alphaValue:255),LWA_ALPHA);
      if (plugin.sctEnabled)
         SetLayeredWindow(hwnd,plugin.sctColor,0,LWA_COLORKEY);
   }
   BBplugin_InvalidateWindow();
}

void BBplugin_Configure (void)
{
   plugin.x=BBplugin_ReadInt("x",100);
   plugin.y=BBplugin_ReadInt("y",100);
   plugin.width=BBplugin_ReadInt("width",100);
   plugin.height=BBplugin_ReadInt("height",100);
   plugin.alphaEnabled=BBplugin_ReadBool("alphaEnabled",false);
   plugin.alphaValue=BBplugin_ReadInt("alphaValue",100);
   plugin.sctEnabled=BBplugin_ReadBool("sctEnabled",false);
   plugin.sctColor=BBplugin_ReadColor("sctColor","#FFFFFF");
   plugin.alwaysOnTop=BBplugin_ReadBool("alwaysOnTop",false);
   plugin.pluginToggle=BBplugin_ReadBool("pluginToggle",false);
   plugin.inSlit=BBplugin_ReadBool("inSlit",false);
      
   plugin.windowStyle=BBplugin_ParseStyle(BBplugin_ReadString("style.window",""));

   // getting style
   BB.font=CreateStyleFont((StyleItem *)GetSettingPtr(SN_TOOLBAR)); 
   plugin.windowStyleItem=(StyleItem *)GetSettingPtr(plugin.windowStyle);

   if (plugin.sctEnabled)
      sctBackground=CreateSolidBrush(plugin.sctColor);

   BBpluginCallback_OnConfigure();
}

bool BBplugin_IsBroam (const char ** broam, const char * t2)
{
   if (!strncmp(*broam,t2,strlen(t2))) {
      *broam+=strlen(t2);
      return(true);
   }
   return(false);
}

// Displays default about dialog box
void BBplugin_AboutBox (void)
{
   char txt[1000];
   sprintf(txt,"%s\ncreated by %s\n%s",pinfo.pVersion,pinfo.pInfoAuthor,pinfo.pInfoLink);
   MessageBox(plugin.hwnd,txt,"About",MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
}

// Executes a command
void BBplugin_Execute (const char * file)
{
   char editor[MAX_PATH];
   GetBlackboxEditor(editor);
   BBExecute(NULL,NULL,editor,file,NULL,SW_SHOWNORMAL,false);
}


// ==========================================================================
// DLL Exported Functions
// ==========================================================================

// Retrieves plugin's info --------------------------------------------------
extern "C" DLL_EXPORT LPCSTR pluginInfo (int field)
{
	switch (field) {
		case PLUGIN_NAME:    return(pinfo.pAppName);       // Plugin name
		case PLUGIN_VERSION: return(pinfo.pInfoVersion);   // Plugin version
		case PLUGIN_AUTHOR:  return(pinfo.pInfoAuthor);    // Author
		case PLUGIN_RELEASE: return(pinfo.pInfoRelDate);   // Release date, preferably in yyyy-mm-dd format
		case PLUGIN_LINK:    return(pinfo.pInfoLink);      // Link to author's website
		case PLUGIN_EMAIL:   return(pinfo.pInfoEmail);     // Author's email
		default:             return(pinfo.pVersion);       // Fallback: Plugin name + version, e.g. "MyPlugin 1.0"
	}
}

// Loads the plugin ---------------------------------------------------------
extern "C" DLL_EXPORT int beginPlugin (HINSTANCE hPluginInstance)
{ 
   if (!BBplugin_LoadPlugin(hPluginInstance)) return(1);

   if (!BBpluginCallback_OnLoadPlugin()) return(1);
   return(0);
}

// Loads the plugin into slit -----------------------------------------------
extern "C" DLL_EXPORT int beginSlitPlugin (HINSTANCE hPluginInstance, HWND hSlit)
{
   BB.slit=hSlit;
   if (!BBplugin_LoadPlugin(hPluginInstance)) return(1);
      
   SendMessage(BB.slit,SLIT_ADD,0,(LPARAM)plugin.hwnd);

   if (!BBpluginCallback_OnLoadPlugin()) return(1);
   return(0);
}

// Loads the plugin into slit -----------------------------------------------
extern "C" DLL_EXPORT int beginPluginEx (HINSTANCE hPluginInstance, HWND hSlit)
{
   BB.slit=hSlit;
   if (!BBplugin_LoadPlugin(hPluginInstance)) return(1);

   if (plugin.inSlit) SendMessage(BB.slit,SLIT_ADD,0,(LPARAM)plugin.hwnd);

   if (!BBpluginCallback_OnLoadPlugin()) return(1);
   return(0);
}

// Unloads the plugin -------------------------------------------------------
extern "C" DLL_EXPORT void endPlugin (HINSTANCE hPluginInstance)
{
   BBpluginCallback_OnUnloadPlugin();
   if (BB.slit) SendMessage(BB.slit,SLIT_REMOVE,0,(LPARAM)plugin.hwnd);
   DestroyWindow(plugin.hwnd);
   UnregisterClass(pinfo.pClassName,plugin.hInstance);
}
