
#ifndef BBPLUGINSDK_H__
#define BBPLUGINSDK_H__

#define _WIN32_WINNT   0x0500

#include "bbSDK\BBApi.h"
#ifdef   GDIPLUS_IN_USE
#include <gdiplus.h>
#endif

#define  szCoreVersion      "1.0.3"

#define  PLUGIN_CLASSNAME   10
#define  PLUGIN_NAMEVER     11

#define  SN_NONE            -1

typedef struct BB_STRUCT {
   HWND       hwnd;
   HFONT      font;
   HWND       slit;
} BB_STRUCT;

typedef struct PLUGIN_STRUCT {
   HINSTANCE  hInstance;
   HWND       hwnd;
   HBITMAP    layout;
   bool       inSlit;
   int        x,y;
   int        width,height;
   char       directory[MAX_PATH],rcfile[MAX_PATH];
   bool       alphaEnabled,sctEnabled;
   unsigned   alphaValue;
   COLORREF   sctColor;
   bool       alwaysOnTop,pluginToggle;
   int        windowStyle;
   StyleItem  *windowStyleItem;
} PLUGIN_STRUCT;

typedef struct PINFO {
   const char 
      *pAppName, *pClassName, *pInfoVersion, *pVersion,
      *pInfoAuthor, *pInfoRelDate, *pInfoLink, *pInfoEmail;
} PINFO;

typedef struct BROAMS {
   char broam_prefix[100],
        menu_readsettings[100],menu_writesettings[100],menu_editsettings[100],
        menu_about[100],
        window_show[100],window_hide[100],window_toggle[100];
} BROAMS;


// Internal BBpluginSDK Functions -------------------------------------------
void BBplugin_FetchInfo (void);
bool BBplugin_LoadPlugin (HINSTANCE);
void BBplugin_UnloadPlugin (void);
LRESULT CALLBACK BBplugin_WindowProc (HWND, UINT, WPARAM, LPARAM);
void BBplugin_OnWMPaint (void);
void BBplugin_OnUpdateWindow (void);
HGDIOBJ BBplugin_PaintLayout (HDC);
void BBplugin_RepaintWindow (void);
void BBplugin_CreateMenu (void);
void BBplugin_UpdateWindowPrefs (void);
void BBplugin_Configure (void);
void BBplugin_SaveConfiguration (void);
HICON BBplugin_LoadIcon (const char *);
#ifdef GDIPLUS_IN_USE
Gdiplus::Bitmap * BBplugin_LoadImage (const char *);
#endif
int BBplugin_ParseStyle (const char *);
void BBplugin_MakeStyleGradient (HDC, RECT *, StyleItem *);
void BBplugin_SetTextColor (HDC, StyleItem *);
void BBplugin_MakeExpression (char *, const char *);
bool BBplugin_ValueExists (const char *);
int BBplugin_ReadInt (const char *, int);
bool BBplugin_ReadBool (const char *, bool);
COLORREF BBplugin_ReadColor (const char *, const char *);
const char * BBplugin_ReadString (const char * field, const char * def);
void BBplugin_ReadStringC (const char *, const char *, char *);
void BBplugin_WriteInt (const char *, int);
void BBplugin_WriteBool (const char *, bool);
void BBplugin_WriteString (const char *, const char *);
bool BBplugin_IsBroam (const char **, const char *);
void BBplugin_Execute (const char *);
void BBplugin_AboutBox (void);

// Exported Functions -------------------------------------------------------
BB_STRUCT * BBpluginExport_GetBB (void);
PLUGIN_STRUCT * BBpluginExport_GetPlugin (void);
const char * BBpluginExport_GetBroamPrefix (void);
HBRUSH * BBpluginExport_GetSctBrush (void);
void BBpluginExport_Repaint (void);
void BBpluginExport_SendBroam (const char *);

// Callback Functions -------------------------------------------------------
const char * BBpluginCallback_InfoQuery (int);
bool BBpluginCallback_OnLoadPlugin();
void BBpluginCallback_OnUnloadPlugin();
bool BBpluginCallback_OnMessage (HWND, UINT, WPARAM, LPARAM);
void BBpluginCallback_OnBroam (const char *);
void BBpluginCallback_OnPaint (HDC);
bool BBpluginCallback_OnCreateMenu (Menu *);
void BBpluginCallback_OnConfigure (void);
void BBpluginCallback_OnSaveConfiguration (void);

#endif
