
#include "BBpluginSDK.h"

BB_STRUCT BB;
PLUGIN_STRUCT plugin;
PINFO pinfo;
BROAMS broams;

HBRUSH sctBackground;

#ifdef GDIPLUS_IN_USE
ULONG_PTR gdiplusToken;   
#endif



// ==========================================================================
// Internal BBpluginSDK functions
// ==========================================================================

// Asks higher level for plugin information ---------------------------------
void BBplugin_FetchInfo (void)
{
   pinfo.pAppName=BBpluginCallback_InfoQuery(PLUGIN_NAME);
   pinfo.pClassName=BBpluginCallback_InfoQuery(PLUGIN_CLASSNAME);
   pinfo.pInfoVersion=BBpluginCallback_InfoQuery(PLUGIN_VERSION);
   pinfo.pInfoAuthor=BBpluginCallback_InfoQuery(PLUGIN_AUTHOR);
   pinfo.pInfoRelDate=BBpluginCallback_InfoQuery(PLUGIN_RELEASE);
   pinfo.pInfoLink=BBpluginCallback_InfoQuery(PLUGIN_LINK);
   pinfo.pInfoEmail=BBpluginCallback_InfoQuery(PLUGIN_EMAIL);
   pinfo.pVersion=BBpluginCallback_InfoQuery(PLUGIN_NAMEVER);

   sprintf(broams.broam_prefix,"@%s.",pinfo.pAppName);
   sprintf(broams.menu_readsettings,"%ssettings.read",broams.broam_prefix);
   sprintf(broams.menu_writesettings,"%ssettings.write",broams.broam_prefix);
   sprintf(broams.menu_editsettings,"%ssettings.edit",broams.broam_prefix);
   sprintf(broams.menu_about,"%sabout",broams.broam_prefix);
   sprintf(broams.window_show,"%sshow",broams.broam_prefix);
   sprintf(broams.window_hide,"%shide",broams.broam_prefix);
   sprintf(broams.window_toggle,"%stoggle",broams.broam_prefix);
}

// Loads the plugin ---------------------------------------------------------
bool BBplugin_LoadPlugin (HINSTANCE hPluginInstance)
{
   BB.hwnd=GetBBWnd();
   plugin.hInstance=hPluginInstance;

   BBplugin_FetchInfo();

   GetModuleFileName(plugin.hInstance,plugin.directory,sizeof(plugin.directory));
   strcpy(plugin.rcfile,plugin.directory);
   *(strrchr(plugin.directory,'\\')+1)='\0';
   strcpy(strrchr(plugin.rcfile,'.')+1,"rc");

#ifdef GDIPLUS_IN_USE
   // Initialize GDI+
   Gdiplus::GdiplusStartupInput gdiplusStartupInput;
   Gdiplus::GdiplusStartup(&gdiplusToken,&gdiplusStartupInput,NULL);
#endif

   WNDCLASS wc;
   ZeroMemory(&wc,sizeof(wc));
   wc.lpfnWndProc=BBplugin_WindowProc;      
   wc.hInstance=plugin.hInstance;    
   wc.lpszClassName=pinfo.pClassName;    
   wc.hCursor=LoadCursor(NULL,IDC_ARROW);
   wc.style=CS_DBLCLKS;

	if (!RegisterClass(&wc)) {
		MessageBox(BB.hwnd,"Error registering window class","Error",
         MB_OK|MB_ICONERROR|MB_TOPMOST);
		return(false);
	}

   plugin.hwnd=CreateWindowEx(
      WS_EX_TOOLWINDOW,
      pinfo.pClassName,
      NULL,
      WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE,
      0,0,
      0,0,
      NULL,
      NULL,
      plugin.hInstance,
      NULL);
   if (plugin.hwnd==NULL) {
      MessageBox(BB.hwnd,"Error creating plugin window","Error",
         MB_OK|MB_ICONERROR|MB_TOPMOST);
      UnregisterClass(pinfo.pClassName,plugin.hInstance);
      return(false);
   }

   if (!BBpluginCallback_OnLoadPlugin()) return(false);

   BBplugin_Configure();
   BBplugin_UpdateWindowPrefs();

   return(true);
}

// Unloads the plugin -------------------------------------------------------
void BBplugin_UnloadPlugin (void)
{
   BBpluginCallback_OnUnloadPlugin();
   if (BB.slit) SendMessage(BB.slit,SLIT_REMOVE,0,(LPARAM)plugin.hwnd);
   DeleteObject(BB.font);
   if (sctBackground) DeleteObject(sctBackground);
   if (plugin.layout) DeleteObject(plugin.layout);
   DestroyWindow(plugin.hwnd);
   UnregisterClass(pinfo.pClassName,plugin.hInstance);
#ifdef GDIPLUS_IN_USE
   Gdiplus::GdiplusShutdown(gdiplusToken);
#endif
}

static int msgs[]={BB_RECONFIGURE,BB_BROADCAST,0};

// Plugin's window procedure ------------------------------------------------
LRESULT CALLBACK BBplugin_WindowProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   if (BBpluginCallback_OnMessage(hwnd,message,wParam,lParam)) return(0);

   switch (message) {
          case WM_CREATE:
               SendMessage(BB.hwnd,BB_REGISTERMESSAGE,(WPARAM)hwnd,(LPARAM)msgs);
               MakeSticky(hwnd);
               break;
          case WM_PAINT: 
               BBplugin_OnWMPaint();
               break;
          
          case WM_LBUTTONDOWN:
               SetFocus(hwnd);         // kvuli ztraceni focusu ve slitu
               if (GetAsyncKeyState(VK_CONTROL) & 0x8000) {
                  PostMessage(hwnd,WM_SYSCOMMAND,0xF012,0);          // start moving
                  return(0);
               }
               if (GetAsyncKeyState(VK_MENU) & 0x8000) {
  				      PostMessage(hwnd,WM_SYSCOMMAND,0xF008,0);          // start sizing
                  return(0);
               }
               //BBplugin_RepaintWindow();
               break;
          case WM_RBUTTONUP:
               BBplugin_CreateMenu();
               break;

          case WM_WINDOWPOSCHANGED: {
               WINDOWPOS *wp=(WINDOWPOS*)lParam;
               if (!BB.slit && !(wp->flags & SWP_NOMOVE)) {
                  plugin.x=wp->x;
                  plugin.y=wp->y;
               }
               if (!(wp->flags & SWP_NOSIZE)) {
                  plugin.width=wp->cx;
                  plugin.height=wp->cy;
                  BBplugin_RepaintWindow();
               }
               break;
          }
          case WM_EXITSIZEMOVE:
               if (BB.slit) SendMessage(BB.slit,SLIT_UPDATE,0,(LPARAM)plugin.hwnd);
               break;
          case WM_CLOSE:
               break;

          case BB_RECONFIGURE:
               BBplugin_Configure();
               BBplugin_UpdateWindowPrefs();
               break;
          case BB_BROADCAST: {
               const char *msg=(LPCSTR)lParam;
               if (BBplugin_IsBroam(&msg,"@BBShowPlugins") && plugin.pluginToggle) ShowWindow(hwnd,SW_SHOW);
               if (BBplugin_IsBroam(&msg,"@BBHidePlugins") && plugin.pluginToggle) ShowWindow(hwnd,SW_HIDE);
               
               if (BBplugin_IsBroam(&msg,broams.window_show)) {
                  ShowWindow(plugin.hwnd,SW_SHOW);
               }
               if (BBplugin_IsBroam(&msg,broams.window_hide)) {
                  ShowWindow(plugin.hwnd,SW_HIDE);
               }
               if (BBplugin_IsBroam(&msg,broams.window_toggle)) {
                  ShowWindow(plugin.hwnd,IsWindowVisible(plugin.hwnd)?SW_HIDE:SW_SHOW);
               }

               if (BBplugin_IsBroam(&msg,broams.menu_readsettings)) {
                  BBplugin_Configure();
                  BBplugin_UpdateWindowPrefs();
               }
               if (BBplugin_IsBroam(&msg,broams.menu_writesettings)) 
                  BBplugin_SaveConfiguration();
               if (BBplugin_IsBroam(&msg,broams.menu_editsettings)) 
                  BBplugin_Execute(plugin.rcfile);
               if (BBplugin_IsBroam(&msg,broams.menu_about)) 
                  BBplugin_AboutBox();

               msg+=strlen(broams.broam_prefix);
               BBpluginCallback_OnBroam(msg);
               break;
          }

          default:
               return(DefWindowProc(hwnd,message,wParam,lParam));
   }
   return(0);
}

// Painting function when WM_PAINT occures ----------------------------------
void BBplugin_OnWMPaint (void)
{
   PAINTSTRUCT ps;
	HDC hdc=BeginPaint(plugin.hwnd,&ps);  

   HDC myDC=CreateCompatibleDC(hdc);

	HGDIOBJ previousBmp;

   if (plugin.layout==NULL) previousBmp=BBplugin_PaintLayout(myDC);
   else previousBmp=SelectObject(myDC,plugin.layout);

   BitBlt(hdc,
      ps.rcPaint.left,ps.rcPaint.top,ps.rcPaint.right-ps.rcPaint.left,ps.rcPaint.bottom-ps.rcPaint.top,
      myDC,
      ps.rcPaint.left,ps.rcPaint.top,
      SRCCOPY);

   SelectObject(myDC,previousBmp);
	DeleteDC(myDC);

   EndPaint(plugin.hwnd,&ps);
}

// Painting function when sct or alpha is enabled ---------------------------
void BBplugin_OnUpdateWindow (void)
{
   HDC screenDC=GetDC(NULL);
   HDC myDC=CreateCompatibleDC(screenDC);

	HGDIOBJ previousBmp=BBplugin_PaintLayout(myDC);

   BLENDFUNCTION blendPixelFunction;
   blendPixelFunction.BlendOp=AC_SRC_OVER;
   blendPixelFunction.BlendFlags=0;
   blendPixelFunction.SourceConstantAlpha=plugin.alphaEnabled?plugin.alphaValue:255;
   blendPixelFunction.AlphaFormat=plugin.alphaEnabled?0:AC_SRC_ALPHA;
   POINT ptSrc={0,0};
   SIZE size={plugin.width,plugin.height};
#ifdef GDIPLUS_IN_USE
   UpdateLayeredWindow(plugin.hwnd,screenDC,0,&size,myDC,&ptSrc,0,&blendPixelFunction,ULW_ALPHA);
#else
   if (plugin.sctEnabled)
      UpdateLayeredWindow(plugin.hwnd,screenDC,0,&size,myDC,&ptSrc,plugin.sctColor,&blendPixelFunction,ULW_COLORKEY);
   else
      UpdateLayeredWindow(plugin.hwnd,screenDC,0,&size,myDC,&ptSrc,0,&blendPixelFunction,ULW_ALPHA);
#endif

   SelectObject(myDC,previousBmp);
   DeleteDC(myDC);
   ReleaseDC(GetDesktopWindow(),screenDC);
}

// Plugin's painting function -----------------------------------------------
HGDIOBJ BBplugin_PaintLayout (HDC myDC)
{
   //plugin.layout=CreateCompatibleBitmap(hdc,plugin.width,plugin.height);
   BITMAPINFO BMInfo;
   ZeroMemory(&BMInfo,sizeof(BITMAPINFO));
   BMInfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
   BMInfo.bmiHeader.biWidth=plugin.width;
   BMInfo.bmiHeader.biHeight=plugin.height;
   BMInfo.bmiHeader.biPlanes=1;
   BMInfo.bmiHeader.biBitCount=32;
   BMInfo.bmiHeader.biCompression=BI_RGB;
   BYTE *rgbaData;
   plugin.layout=CreateDIBSection(myDC,&BMInfo,DIB_RGB_COLORS,(void **)&rgbaData,NULL,0);

   HGDIOBJ previousBmp=SelectObject(myDC,plugin.layout);

   RECT rect;
   SetRect(&rect,0,0,plugin.width,plugin.height);
   if (!plugin.sctEnabled) 
      BBplugin_MakeStyleGradient(myDC,&rect,plugin.windowStyleItem);
#ifndef GDIPLUS_IN_USE
   else 
      FillRect(myDC,&rect,sctBackground);
#endif

   HGDIOBJ oldfont=SelectObject(myDC,BB.font);
   SetBkMode(myDC,TRANSPARENT);
   BBplugin_SetTextColor(myDC,plugin.windowStyleItem);
      
   BBpluginCallback_OnPaint(myDC);

   SelectObject(myDC,oldfont);

   return(previousBmp);
}

// Forces the plugin's window to repaint ------------------------------------
void BBplugin_RepaintWindow (void) 
{
   if (plugin.layout) {
      DeleteObject(plugin.layout);
      plugin.layout=NULL;
   }
   if (!plugin.alphaEnabled && !plugin.sctEnabled)
      InvalidateRect(plugin.hwnd,NULL,false);
   else
      // WS_EX_LAYERED window never sends WM_PAINT
      BBplugin_OnUpdateWindow();
}

// Creates and displays plugin's menu ---------------------------------------
void BBplugin_CreateMenu (void)
{
   Menu *mainMenu,*settingsMenu;
   char id[1000];

   sprintf(id,"%s_IDMain",pinfo.pAppName);
   mainMenu=MakeNamedMenu(pinfo.pAppName,id,true);

   if (BBpluginCallback_OnCreateMenu(mainMenu))
      MakeMenuNOP(mainMenu,NULL);

   sprintf(id,"%s_IDSettings",pinfo.pAppName);
   settingsMenu=MakeNamedMenu("Settings",id,true);
   MakeMenuItem(settingsMenu,"read settings",broams.menu_readsettings,false);
   MakeMenuItem(settingsMenu,"write settings",broams.menu_writesettings,false);
   MakeMenuItem(settingsMenu,"edit settings",broams.menu_editsettings,false);
   MakeSubmenu(mainMenu,settingsMenu,"Settings");
   MakeMenuItem(mainMenu,"About",broams.menu_about,false);   
   
   ShowMenu(mainMenu);
}

// Updates plugin's window preferences --------------------------------------
void BBplugin_UpdateWindowPrefs (void)
{
   SetWindowPos(plugin.hwnd,plugin.alwaysOnTop?HWND_TOPMOST:HWND_NOTOPMOST,
                plugin.x,plugin.y,plugin.width,plugin.height,SWP_NOACTIVATE|SWP_NOSENDCHANGING);
   long exstyle=WS_EX_TOOLWINDOW;
   if ((!BB.slit || !plugin.inSlit) && (plugin.sctEnabled || plugin.alphaEnabled)) exstyle|=WS_EX_LAYERED;
   SetWindowLong(plugin.hwnd,GWL_EXSTYLE,exstyle);
   BBplugin_RepaintWindow();
}

// Configures the plugin ----------------------------------------------------
void BBplugin_Configure (void)
{
   plugin.x=BBplugin_ReadInt("x",100);
   plugin.y=BBplugin_ReadInt("y",100);
   plugin.width=BBplugin_ReadInt("width",100);
   plugin.height=BBplugin_ReadInt("height",100);
   plugin.alphaEnabled=BBplugin_ReadBool("alphaEnabled",false);
   plugin.alphaValue=BBplugin_ReadInt("alphaValue",100);
   plugin.sctEnabled=BBplugin_ReadBool("sctEnabled",false);
   plugin.sctColor=BBplugin_ReadColor("sctColor","#FF00FF");
   plugin.alwaysOnTop=BBplugin_ReadBool("alwaysOnTop",false);
   plugin.pluginToggle=BBplugin_ReadBool("pluginToggle",false);
   plugin.inSlit=BBplugin_ReadBool("inSlit",false);
      
   plugin.windowStyle=BBplugin_ParseStyle(BBplugin_ReadString("style.window",""));

   // getting style
   if (BB.font) DeleteObject(BB.font);
   BB.font=CreateStyleFont((StyleItem *)GetSettingPtr(SN_TOOLBAR)); 
   plugin.windowStyleItem=(StyleItem *)GetSettingPtr(plugin.windowStyle);

   if (BB.slit && plugin.inSlit) plugin.alphaEnabled=plugin.sctEnabled=false;

   if (plugin.sctEnabled) {
      if (sctBackground) DeleteObject(sctBackground);
      sctBackground=CreateSolidBrush(plugin.sctColor);
   }

   BBpluginCallback_OnConfigure();
}

// Saves plugin's configuration ---------------------------------------------
void BBplugin_SaveConfiguration (void)
{
   BBplugin_WriteInt("x",plugin.x);
   BBplugin_WriteInt("y",plugin.y);
   BBplugin_WriteInt("width",plugin.width);
   BBplugin_WriteInt("height",plugin.height);

   BBpluginCallback_OnSaveConfiguration();
}

// Loads an icon from file --------------------------------------------------
HICON BBplugin_LoadIcon (const char * path)
{
   HICON icon=NULL;
   if (path!=NULL) {
      char dirpath[MAX_PATH];
      sprintf(dirpath,"%s%s",plugin.directory,path);
      icon=(HICON)LoadImage(0,dirpath,IMAGE_ICON,0,0,LR_LOADFROMFILE);
      if (icon==NULL) 
         MessageBox(BB.hwnd,dirpath,"Cannot load icon",MB_OK|MB_ICONERROR|MB_TOPMOST);
   }
   return(icon);
}

#ifdef GDIPLUS_IN_USE
// Loads an image from file -------------------------------------------------
Gdiplus::Bitmap * BBplugin_LoadImage (const char * path)
{
   Gdiplus::Bitmap * bitmap=NULL;
   if (path!=NULL) {
      char dirpath[MAX_PATH];
      sprintf(dirpath,"%s%s",plugin.directory,path);

      HICON icon=(HICON)LoadImage(0,dirpath,IMAGE_ICON,0,0,LR_LOADFROMFILE);
      if (icon) {
         bitmap=new Gdiplus::Bitmap(icon);
         DestroyIcon(icon);
      }
      else {
         WCHAR file[MAX_PATH*2];
         MultiByteToWideChar(CP_ACP,0,dirpath,-1,file,MAX_PATH*2);
         bitmap=new Gdiplus::Bitmap(file);
      }

      if (bitmap==NULL || bitmap->GetLastStatus()!=Gdiplus::Ok) 
         MessageBox(BB.hwnd,dirpath,"Cannot load image",MB_OK|MB_ICONERROR|MB_TOPMOST);
   }

   return(bitmap);
}
#endif

// Parses style expression --------------------------------------------------
int BBplugin_ParseStyle (const char * styletxt)
{
   int style=SN_NONE;
   if (!stricmp(styletxt,"toolbar")) style=SN_TOOLBAR;
   if (!stricmp(styletxt,"button")) style=SN_TOOLBARBUTTON;
   if (!stricmp(styletxt,"buttonpressed")) style=SN_TOOLBARBUTTONP;
   if (!stricmp(styletxt,"label")) style=SN_TOOLBARLABEL;
   if (!stricmp(styletxt,"windowlabel")) style=SN_TOOLBARWINDOWLABEL;
   if (!stricmp(styletxt,"clock")) style=SN_TOOLBARCLOCK;
   return(style);
}

// Fills RECT with style gradient with respect to the NULL style ------------
void BBplugin_MakeStyleGradient (HDC hdc, RECT * rect, StyleItem * style)
{
   if (style) MakeStyleGradient(hdc,rect,style,true);
}

// Sets a color with respect to the NULL style ------------------------------
void BBplugin_SetTextColor (HDC hdc, StyleItem * style)
{
   if (style) SetTextColor(hdc,style->TextColor);
}

// Reading and writing .rc items --------------------------------------------
void BBplugin_MakeExpression (char * expr, const char * field)
{
   sprintf(expr,"%s.%s:",pinfo.pAppName,field);
}

bool BBplugin_ValueExists (const char * field)
{
   const char *txt="ABCDEF";
   return(strcmp(txt,BBplugin_ReadString(field,txt))!=0);
}

int BBplugin_ReadInt (const char * field, int def)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   return(ReadInt(plugin.rcfile,expr,def));
}
bool BBplugin_ReadBool (const char * field, bool def)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   return(ReadBool(plugin.rcfile,expr,def));
}
COLORREF BBplugin_ReadColor (const char * field, const char * def)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   return(ReadColor(plugin.rcfile,expr,def));
}
const char * BBplugin_ReadString (const char * field, const char * def)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   return(ReadString(plugin.rcfile,expr,def));
}
void BBplugin_ReadStringC (const char * field, const char * def, char * target)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   const char * value=ReadString(plugin.rcfile,expr,def);
   if (value) strcpy(target,value);
}
void BBplugin_WriteInt (const char * field, int value)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   WriteInt(plugin.rcfile,expr,value);
}
void BBplugin_WriteBool (const char * field, bool value)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   WriteBool(plugin.rcfile,expr,value);
}
void BBplugin_WriteString (const char * field, const char * value)
{
   char expr[1000];
   BBplugin_MakeExpression(expr,field);
   WriteString(plugin.rcfile,expr,value);
}

// Checks broams equivalence ------------------------------------------------
bool BBplugin_IsBroam (const char ** broam, const char * t2)
{
   if (!strncmp(*broam,t2,strlen(t2))) {
      *broam+=strlen(t2);
      return(true);
   }
   return(false);
}

// Executes a command -------------------------------------------------------
void BBplugin_Execute (const char * file)
{
   char editor[MAX_PATH];
   GetBlackboxEditor(editor);
   BBExecute(NULL,NULL,editor,file,NULL,SW_SHOWNORMAL,false);
}

// Displays default about box -----------------------------------------------
void BBplugin_AboutBox (void)
{
   char txt[1000];
   sprintf(txt,"%s\t\n%s\n(core version: %s)\n\ncreated by %s\n%s",
      pinfo.pVersion,pinfo.pInfoRelDate,szCoreVersion,pinfo.pInfoAuthor,pinfo.pInfoLink);
   MessageBox(plugin.hwnd,txt,"About",MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
}


// ==========================================================================
// Exported BBpluginSDK Functions
// ==========================================================================

// Returns pointer to the "BB" structure ------------------------------------
BB_STRUCT * BBpluginExport_GetBB (void)
{
   return(&BB);
}

// Returns pointer to the "plugin" structure --------------------------------
PLUGIN_STRUCT * BBpluginExport_GetPlugin (void)
{
   return(&plugin);
}

// Returns broam prefix for this plugin -------------------------------------
const char * BBpluginExport_GetBroamPrefix (void)
{
   return(broams.broam_prefix);
}

// Returns single color transparency brush ----------------------------------
HBRUSH * BBpluginExport_GetSctBrush (void)
{
   return(&sctBackground);
}

// Forces repainting of the window ------------------------------------------
void BBpluginExport_Repaint (void)
{
   BBplugin_RepaintWindow();
}

// Sends a bro@m ------------------------------------------------------------
void BBpluginExport_SendBroam (const char * broam)
{
   SendMessage(BB.hwnd,BB_BROADCAST,0,(LPARAM)broam);
}


// ==========================================================================
// DLL Exported Functions
// ==========================================================================

// Retrieves plugin's info --------------------------------------------------
extern "C" DLL_EXPORT LPCSTR pluginInfo (int field)
{
	switch (field) {
		case PLUGIN_NAME:    return(pinfo.pAppName);       // Plugin name
		case PLUGIN_VERSION: return(pinfo.pInfoVersion);   // Plugin version
		case PLUGIN_AUTHOR:  return(pinfo.pInfoAuthor);    // Author
		case PLUGIN_RELEASE: return(pinfo.pInfoRelDate);   // Release date, preferably in yyyy-mm-dd format
		case PLUGIN_LINK:    return(pinfo.pInfoLink);      // Link to author's website
		case PLUGIN_EMAIL:   return(pinfo.pInfoEmail);     // Author's email
		default:             return(pinfo.pVersion);       // Fallback: Plugin name + version, e.g. "MyPlugin 1.0"
	}
}

// Loads the plugin ---------------------------------------------------------
extern "C" DLL_EXPORT int beginPlugin (HINSTANCE hPluginInstance)
{ 
   if (!BBplugin_LoadPlugin(hPluginInstance)) return(1);

   //if (!BBpluginCallback_OnLoadPlugin()) return(1);
   return(0);
}

// Loads the plugin into slit -----------------------------------------------
extern "C" DLL_EXPORT int beginSlitPlugin (HINSTANCE hPluginInstance, HWND hSlit)
{
   BB.slit=hSlit;
   if (!BBplugin_LoadPlugin(hPluginInstance)) return(1);
      
   SendMessage(BB.slit,SLIT_ADD,0,(LPARAM)plugin.hwnd);

   //if (!BBpluginCallback_OnLoadPlugin()) return(1);
   return(0);
}

// Loads the plugin into slit -----------------------------------------------
extern "C" DLL_EXPORT int beginPluginEx (HINSTANCE hPluginInstance, HWND hSlit)
{
   BB.slit=hSlit;
   if (!BBplugin_LoadPlugin(hPluginInstance)) return(1);

   if (plugin.inSlit) SendMessage(BB.slit,SLIT_ADD,0,(LPARAM)plugin.hwnd);

   //if (!BBpluginCallback_OnLoadPlugin()) return(1);
   return(0);
}

// Unloads the plugin -------------------------------------------------------
extern "C" DLL_EXPORT void endPlugin (HINSTANCE hPluginInstance)
{
   BBplugin_UnloadPlugin();
}
