
#ifndef GRAPHS_H__
#define GRAPHS_H__

#include "BBpluginSDK.h"
#ifdef GDIPLUS_IN_USE
#include <gdiplus.h>
#endif

#define DATATYPE      __int64
#define TOMEM(m,o,x)  (* ( (DATATYPE *) ((unsigned char *)m+(o)*sizeof(DATATYPE)) ) )=x
#define FROMMEM(m,o)  (* ( (DATATYPE *) ((unsigned char *)m+(o)*sizeof(DATATYPE)) ) )

class Graph
{
   private:
      int x,y,width,height;
      COLORREF graphcolor,textcolor,backgroundcolor;
      StyleItem *background;
#ifdef GDIPLUS_IN_USE
      Gdiplus::Pen *hPen;
      Gdiplus::SolidBrush *solidBackground;
#else
      HPEN hPen;
      HBRUSH solidBackground;
#endif
      DATATYPE maxValue;
      void *data;
      char displayText[50];
      bool visible;
      int borderWidth;
      bool setFixedMaxVal;
      long datasize;

   public:
      Graph (const char * graphrc) {
         char str[1000];
         sprintf(str,"%s.visible",graphrc);
         visible=BBplugin_ReadBool(str,false);
         sprintf(str,"%s.x",graphrc);
         x=BBplugin_ReadInt(str,10);
         sprintf(str,"%s.y",graphrc);
         y=BBplugin_ReadInt(str,10);
         sprintf(str,"%s.width",graphrc);
         width=BBplugin_ReadInt(str,100);
         sprintf(str,"%s.height",graphrc);
         height=BBplugin_ReadInt(str,100);
         sprintf(str,"%s.style",graphrc);
         int style=BBplugin_ParseStyle(BBplugin_ReadString(str,""));
         background=(StyleItem *)GetSettingPtr(style);
         sprintf(str,"%s.color",graphrc);
         if (BBplugin_ValueExists(str)) graphcolor=BBplugin_ReadColor(str,"");
         else
            if (background) graphcolor=background->TextColor;
         sprintf(str,"%s.textColor",graphrc);
         if (BBplugin_ValueExists(str)) textcolor=BBplugin_ReadColor(str,"");
         else
            if (background) textcolor=background->TextColor;
         sprintf(str,"%s.backgroundColor",graphrc);
         if (BBplugin_ValueExists(str)) backgroundcolor=BBplugin_ReadColor(str,"");
         else
            backgroundcolor=-1;

         borderWidth=*(int *)GetSettingPtr(SN_BORDERWIDTH);
         if (borderWidth<1) borderWidth=1;
         if (backgroundcolor!=-1) borderWidth=0;
         datasize=width-2*borderWidth;
         data=LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT,datasize*sizeof(DATATYPE));
#ifdef GDIPLUS_IN_USE
         Gdiplus::Color c1(255,GetRValue(graphcolor),GetGValue(graphcolor),GetBValue(graphcolor));
         hPen=new Gdiplus::Pen(c1,1);
         Gdiplus::Color c2(255,GetRValue(backgroundcolor),GetGValue(backgroundcolor),GetBValue(backgroundcolor));
         solidBackground=(backgroundcolor!=-1)?new Gdiplus::SolidBrush(c2):NULL;
#else
         hPen=CreatePen(PS_SOLID,1,graphcolor);
         solidBackground=(backgroundcolor!=-1)?CreateSolidBrush(backgroundcolor):NULL;
#endif
         setFixedMaxVal=false;
         maxValue=0;
      }
      ~Graph (void) {
#ifdef GDIPLUS_IN_USE
         delete hPen;
         delete solidBackground;
#else
         DeleteObject(hPen);
         DeleteObject(solidBackground);
#endif
         LocalFree(data);
      }
      void addValue (DATATYPE value) {
         DATATYPE max=0;
         for (int i=0;i<datasize-1;i++) {
             DATATYPE x=FROMMEM(data,i+1);
             TOMEM(data,i,x);
             if (x>max) max=x;
         }
         TOMEM(data,datasize-1,value);
         if (value>max) max=value;
         if (!setFixedMaxVal) maxValue=max;
      }
#ifdef GDIPLUS_IN_USE
      void paint (Gdiplus::Graphics & g, HDC hdc, Gdiplus::Font & font) {
         if (visible) {
            Gdiplus::RectF rect(x,y,width,height);
            if (solidBackground) g.FillRectangle(solidBackground,rect);
            else
               if (background) {
                  RECT r;
                  SetRect(&r,x,y,x+width,y+height);
                  MakeStyleGradient(hdc,&r,background,true);
               }

            double unit=(double)(height-2*borderWidth)/(double)maxValue;
            for (int i=0;i<datasize;i++) {
                int v=(int)(unit*(double)FROMMEM(data,i));
                if (v)
                   g.DrawLine(hPen,
                           x+borderWidth+i,
                           y+height-borderWidth-v,
                           x+borderWidth+i,
                           y+height-1-borderWidth);
            }

            Gdiplus::Color c(255,GetRValue(textcolor),GetGValue(textcolor),GetBValue(textcolor));
            Gdiplus::SolidBrush brush(c);
            Gdiplus::StringFormat sf;
            sf.SetAlignment(Gdiplus::StringAlignmentCenter);
            sf.SetLineAlignment(Gdiplus::StringAlignmentCenter);
            WCHAR string[100];
            MultiByteToWideChar(CP_ACP,0,displayText,-1,string,200);
            g.DrawString(string,-1,&font,rect,&sf,&brush);
         }
      }
#else
      void paint (HDC hdc) {
         if (visible) {
            RECT rect;
            SetRect(&rect,x,y,x+width,y+height);
            if (solidBackground) FillRect(hdc,&rect,solidBackground);
            else
               if (background) MakeStyleGradient(hdc,&rect,background,true);

            HGDIOBJ oldPen=SelectObject(hdc,hPen);
            double unit=(double)(height-2*borderWidth)/(double)maxValue;
            for (int i=0;i<datasize;i++) {
                Rectangle(hdc,
                          x+borderWidth+i,
                          y+height-borderWidth-(int)(unit*(double)FROMMEM(data,i)),
                          x+borderWidth+1+i,
                          y+height-borderWidth);
            }
            SelectObject(hdc,oldPen);

            COLORREF previousColor=SetTextColor(hdc,textcolor);
            SetTextAlign(hdc,TA_TOP | TA_LEFT); // ta trubka to jinak kresli uplne blbe...
            DrawText(hdc,displayText,-1,&rect,DT_CENTER|DT_VCENTER|DT_END_ELLIPSIS|DT_SINGLELINE);
            SetTextColor(hdc,previousColor);
         }
      }
#endif
      void reset (void) {
         ZeroMemory(data,datasize*sizeof(DATATYPE));
      }
      char * getDisplayTextPtr (void) {
         return(displayText);
      }
      void setFixedMaxValue (DATATYPE value) {
         setFixedMaxVal=true;
         maxValue=value;
      }
};

#endif