
#ifndef LABELS_H__
#define LABELS_H__

#include "BBpluginSDK.h"
#include <vector>
#ifdef GDIPLUS_IN_USE
#include <gdiplus.h>
#endif

#define  LABEL_MAX   1024

#define  ALIGN_CENTER   TA_CENTER
#define  ALIGN_LEFT     TA_LEFT
#define  ALIGN_RIGHT    TA_RIGHT

class Label
{
   private:
      char  id[LABEL_MAX],caption[LABEL_MAX],content[LABEL_MAX];
#ifdef GDIPLUS_IN_USE
      Gdiplus::Bitmap * icon;
#else
      HICON icon;
#endif

   protected:
      int parseAlign (const char * align) {
         int a=ALIGN_LEFT;
         if (!stricmp(align,"center")) a=ALIGN_CENTER;
         if (!stricmp(align,"right")) a=ALIGN_RIGHT;
         return(a);
      }

   public:
      int captionX,captionY,contentX,contentY,iconX,iconY;
      int captionAlign,contentAlign;
      bool visible;

      Label (Label *l) {
         strcpy(id,l->getID());
         strcpy(caption,l->getCaption());
         captionX=l->captionX;
         captionY=l->captionY;
         contentX=l->contentX;
         contentY=l->contentY;
         iconX=l->iconX;
         iconY=l->iconY;
         captionAlign=l->captionAlign;
         contentAlign=l->contentAlign;
         visible=l->visible;
#ifdef GDIPLUS_IN_USE
         Gdiplus::Bitmap *ic=l->getIcon();
         if (ic) icon=ic->Clone(0,0,ic->GetWidth(),ic->GetHeight(),PixelFormat32bppARGB);
         else icon=NULL;
#else
         icon=l->getIcon();
#endif

         memset(content,0,LABEL_MAX);
      }
      Label (const char *labelrc) {
         char str[1000];
         sprintf(str,"%s.caption.x",labelrc);
         captionX=BBplugin_ReadInt(str,30);
         sprintf(str,"%s.caption.y",labelrc);
         captionY=BBplugin_ReadInt(str,10);
         sprintf(str,"%s.content.x",labelrc);
         contentX=BBplugin_ReadInt(str,50);
         sprintf(str,"%s.content.y",labelrc);
         contentY=BBplugin_ReadInt(str,10);
         sprintf(str,"%s.icon.x",labelrc);
         iconX=BBplugin_ReadInt(str,10);
         sprintf(str,"%s.icon.y",labelrc);
         iconY=BBplugin_ReadInt(str,10);
         sprintf(str,"%s.id",labelrc);
         strcpy(id,BBplugin_ReadString(str,""));
         sprintf(str,"%s.caption",labelrc);
         strcpy(caption,BBplugin_ReadString(str,""));
         sprintf(str,"%s.caption.align",labelrc);
         captionAlign=parseAlign(BBplugin_ReadString(str,""));
         sprintf(str,"%s.content.align",labelrc);
         contentAlign=parseAlign(BBplugin_ReadString(str,""));
         sprintf(str,"%s.icon.path",labelrc);
#ifdef GDIPLUS_IN_USE
         icon=BBplugin_LoadImage(BBplugin_ReadString(str,NULL));
#else
         icon=BBplugin_LoadIcon(BBplugin_ReadString(str,NULL));
#endif
         sprintf(str,"%s.visible",labelrc);
         visible=BBplugin_ReadBool(str,false);

         memset(content,0,LABEL_MAX);
      }

#ifdef GDIPLUS_IN_USE
      void paint (Gdiplus::Graphics & g, int x, int y, Gdiplus::Font & font, Gdiplus::Brush & brush) {
         if (visible) {
            if (icon) g.DrawImage(icon,x+iconX,y+iconY,icon->GetWidth(),icon->GetHeight());

            WCHAR string[LABEL_MAX*2];
            Gdiplus::StringFormat sf;
            Gdiplus::StringAlignment sa;
            int left;
            left=x+captionX;
            switch (captionAlign) {
                   case ALIGN_CENTER: sa=Gdiplus::StringAlignmentCenter; left-=100; break;
                   case ALIGN_LEFT:   sa=Gdiplus::StringAlignmentNear; break;
                   case ALIGN_RIGHT:  sa=Gdiplus::StringAlignmentFar; left-=200; break;
            }
            sf.SetAlignment(sa);
            MultiByteToWideChar(CP_ACP,0,caption,-1,string,LABEL_MAX*2);
            Gdiplus::RectF rect1(left,y+captionY,200,100);
            g.DrawString(string,-1,&font,rect1,&sf,&brush);

            left=x+contentX;
            switch (contentAlign) {
                   case ALIGN_CENTER: sa=Gdiplus::StringAlignmentCenter; left-=100; break;
                   case ALIGN_LEFT:   sa=Gdiplus::StringAlignmentNear; break;
                   case ALIGN_RIGHT:  sa=Gdiplus::StringAlignmentFar; left-=200; break;
            }
            sf.SetAlignment(sa);
            MultiByteToWideChar(CP_ACP,0,content,-1,string,LABEL_MAX*2);
            Gdiplus::RectF rect2(left,y+contentY,200,100);
            g.DrawString(string,-1,&font,rect2,&sf,&brush);
         }
      }
      void paint (Gdiplus::Graphics & g, Gdiplus::Font & font, Gdiplus::Brush & brush) {
         paint(g,0,0,font,brush);
      }
      ~Label (void)                        { delete icon; }
      Gdiplus::Bitmap * getIcon (void)     { return(icon); }
#else
      void paint (HDC hdc, int x, int y) {
         if (visible) {
            if (icon) DrawState(hdc,0,0,(long)icon,0,x+iconX,y+iconY,0,0,DST_ICON);
            SetTextAlign(hdc,TA_TOP | captionAlign);
            TextOut(hdc,x+captionX,y+captionY,caption,strlen(caption));
            SetTextAlign(hdc,TA_TOP | contentAlign);
            TextOut(hdc,x+contentX,y+contentY,content,strlen(content));
         }
      }
      void paint (HDC hdc) {
         paint(hdc,0,0);
      }
      ~Label (void)                        { DestroyIcon(icon); }
      HICON getIcon (void)                 { return(icon); }
#endif
         
      const char * getID (void)            { return(id); }
      const char * getCaption (void)       { return(caption); }
      const char * getContent (void)       { return(content); }
      char * getContentPtr (void)          { return(content); }
};

class LabelList
{
   private:
      std::vector <Label *> items;
   public:
      LabelList (void) { }
      LabelList (LabelList * ll) {
         for (int n=0;n<ll->getNumLabels();n++) {
             Label *l=ll->getLabel(n);
             addLabel(new Label(l));
         }
      }
      ~LabelList (void)                   { clear(); }
      void clear (void)                   { for (int n=0;n<items.size();n++) delete items[n]; items.clear(); }
      void addLabel (Label * l)           { items.insert(items.end(),l); }
      int getNumLabels (void)             { return(items.size()); }
      Label * getLabel (int n)            { return(items[n]); }
#ifdef GDIPLUS_IN_USE
      void paintLabels (Gdiplus::Graphics & g, int x, int y, Gdiplus::Font & font, Gdiplus::Brush & brush) {
         for (int n=0;n<items.size();n++) items[n]->paint(g,x,y,font,brush);
      }
      void paintLabels (Gdiplus::Graphics & g, Gdiplus::Font & font, Gdiplus::Brush & brush) {
         paintLabels(g,0,0,font,brush);
      }
#else
      void paintLabels (HDC hdc, int x, int y) { 
         for (int n=0;n<items.size();n++) items[n]->paint(hdc,x,y); 
      }
      void paintLabels (HDC hdc) { 
         paintLabels(hdc,0,0); 
      }
#endif
};

#endif