
#include "BBpopup.h"

PLUGIN_STRUCT *plugin;

struct plugindata {
   int timer;
   bool autoHide;
   bool playSound;
   char soundFile[MAX_PATH];
} plugindata;

UINT timer;

EventList list;
int show_x,size_x;
bool showing=false;


const char * BBpluginCallback_InfoQuery (int field)
{
	switch (field) {
          case PLUGIN_NAME:      return(szAppName);
          case PLUGIN_CLASSNAME: return(szClassName);
          case PLUGIN_VERSION:   return(szInfoVersion);  
          case PLUGIN_AUTHOR:    return(szInfoAuthor);   
          case PLUGIN_RELEASE:   return(szInfoRelDate); 
          case PLUGIN_LINK:      return(szInfoLink);   
          case PLUGIN_EMAIL:     return(szInfoEmail);   
          case PLUGIN_NAMEVER:   return(szVersion);
   }
   return(NULL);
}

bool BBpluginCallback_OnLoadPlugin()
{ 
   plugin=BBpluginExport_GetPlugin();

   return(true);
}

void BBpluginCallback_OnUnloadPlugin()
{
   KillTimer(NULL,timer);
}

bool BBpluginCallback_OnMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg) {          
          case WM_LBUTTONDOWN:
               if (showing) {
                  HWND h=list.getItem()->getHwnd();
                  if (IsWindow(h)) {
                     //ShowWindow(h,SW_SHOW);
                     SetWindowPos(h,HWND_TOP,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
                     SetFocus(hwnd);
                  }
               }
               break;
          case WM_RBUTTONUP:
               playNext();
   }

   return(false);
}

void BBpluginCallback_OnBroam (const char * broam)
{
   if (BBplugin_IsBroam(&broam,ADDEVENT_COMMAND)) {
      char *p=strtok((char *)broam+1," ");
      if (!p) return;
      HWND hwnd=(HWND)atol(p);
      broam+=strlen(p)+2;
      if (*broam!='"') return;
      p=strtok((char *)broam,"\"");
      if (!p) return;
      list.addItem(new EventItem(hwnd,p));   
      if (!showing) playNext();
   }
}

void BBpluginCallback_OnPaint (HDC hdc)
{
   RECT rect;

   EventItem *item=list.getItem();
   if (item) {
      SetRect(&rect,show_x,0,plugin->width,plugin->height);
      const char *text=item->getText();
      DrawText(hdc,text,-1,&rect,DT_LEFT | DT_VCENTER | DT_SINGLELINE);
   }
}

bool BBpluginCallback_OnCreateMenu (Menu * mainMenu)
{
   return(false);
}

void BBpluginCallback_OnConfigure (void)
{
   plugindata.timer=BBplugin_ReadInt("timer",1000);
   plugindata.autoHide=BBplugin_ReadBool("autoHide",false);
   plugindata.playSound=BBplugin_ReadBool("playSound",false);
   const char *sndfile=BBplugin_ReadString("soundFile",NULL);
   strcpy(plugindata.soundFile,sndfile);

   toggleWindow(!plugindata.autoHide);
}

void BBpluginCallback_OnSaveConfiguration (void)
{
}


TIMERPROC TimerProc (void)
{
   if (show_x>-size_x) show_x--;
   else {
      playNext();
   }
   BBpluginExport_Repaint();

   return(0);
}

void playNext (void)
{
   if (showing) {
      KillTimer(NULL,timer);
      list.removeItem();
      BBpluginExport_Repaint();
      if (plugindata.autoHide) toggleWindow(false);
      showing=false;
   }
   if (list.getNumItems()) {
      show_x=plugin->width;
      size_x=getStringWidth(list.getItem()->getText());
      timer=SetTimer(NULL,NULL,plugindata.timer,(TIMERPROC)TimerProc);
      if (plugindata.autoHide) toggleWindow(true);
      showing=true;
      if (plugindata.playSound) {
         char dirpath[MAX_PATH];
         sprintf(dirpath,"%s%s",plugin->directory,plugindata.soundFile);
         PlaySound(access(plugindata.soundFile,0)==0?plugindata.soundFile:dirpath,NULL,SND_FILENAME | SND_ASYNC | SND_NODEFAULT);
      }
   }
}

void toggleWindow (bool show)
{
   BB_STRUCT *BB=BBpluginExport_GetBB();
   ShowWindow(plugin->hwnd,show?SW_SHOW:SW_HIDE);
   if (BB->slit && plugin->inSlit) {
      SendMessage(BB->slit,show?SLIT_ADD:SLIT_REMOVE,0,(LPARAM)plugin->hwnd);
   }
}

int getStringWidth (const char * str)
{
	RECT rect={0,0,0,0};
	HDC hdc=CreateCompatibleDC(NULL);
	HGDIOBJ oldfont=SelectObject(hdc,BBpluginExport_GetBB()->font);
	DrawText(hdc,str,-1,&rect,DT_LEFT | DT_CALCRECT);

	SelectObject(hdc,oldfont);
	DeleteDC(hdc);
	
	return(rect.right);
}

