
#include "ActionList.h"

// A C T I O N  class --------------------------------------------------------

Action::Action (bool a, short t, int h, int mi, int d, int m, int y, const char * s,
                const char * p, const char * r)
{
   active=a;
   type=t;
   day=d;
   month=m;
   year=y;
   hour=h;
   minute=mi;
   text=s;
   program=p;
   params=r;
   countHash();
}

long Action::getID (void)
{
   return(id);
}

void Action::countHash (void)
{
   /*SYSTEMTIME st;
   ZeroMemory(&st,sizeof(SYSTEMTIME));
   st.wHour=hour;
   st.wMinute=minute;
   st.wDay=day;
   st.wMonth=month;
   st.wYear=year;


   FILETIME ft;
   SystemTimeToFileTime(&st,&ft);
   ULARGE_INTEGER li;
   memcpy(&li,&ft,sizeof(FILETIME));
   hash=li.QuadPart;*/
   
   struct tm time;
   memset(&time,0,sizeof(tm));
   time.tm_hour=hour;
   time.tm_min=minute;
   time.tm_mday=day;
   time.tm_mon=month-1;
   time.tm_year=year-1900;
   hash=mktime(&time);
}

time_t Action::getHash (void)
{
   return(hash);
}

const char * Action::getText (void)
{
   return(text.c_str());
}

short Action::getType (void)
{
   return(type);
}

int Action::getHour (void)
{
   return(hour);
}
int Action::getMinute (void)
{
   return(minute);
}
int Action::getDay (void)
{
   return(day);
}
int Action::getMonth (void)
{
   return(month);
}
int Action::getYear (void)
{
   return(year);
}
const char * Action::getProgram (void)
{
   return(program.c_str());
}
const char * Action::getParams (void)
{
   return(params.c_str());
}
bool Action::isActive (void)
{
   return(active);
}

void Action::setID (long i)
{
   id=i;
}
void Action::setText (const char * t)
{
   text=t;
}
void Action::setType (short t)
{
   type=t;
}

void Action::setHour (int h)
{
   hour=h;
   countHash();
}
void Action::setMinute (int m)
{
   minute=m;
   countHash();
}
void Action::setDay (int d)
{
   day=d;
   countHash();
}
void Action::setMonth (int m)
{
   month=m;
   countHash();
}
void Action::setYear (int y)
{
   year=y;
   countHash();
}
void Action::setProgram (const char * p)
{
   program=p;
}
void Action::setParams (const char * p)
{
   params=p;
}
void Action::setActive (bool b)
{
   active=b;
}
void Action::toggleActive (void)
{
   active=!active;
}

// A C T I O N L I S T class -------------------------------------------------

ActionList::ActionList (void)
{
   availableID=0;
}

int ActionList::addObject (Action * action)
{
   action->setID((availableID++));
   items.insert(items.end(),action);
   return(getNumObjects()-1);          // prida se na posledni misto
}

Action * ActionList::getObject (int num)
{
   return(items[num]);
}

void ActionList::removeObject (int num)
{
   delete items[num];
   items.erase(items.begin()+num);
}

void ActionList::removeAll (void)
{
   for (int i=0;i<items.size();i++) delete items[i];
   items.clear();
}

int ActionList::getNumObjects (void)
{
   return(items.size());
}

void ActionList::sort (int * index)
{   
   long id=-1;
   if (*index!=-1) id=items[*index]->getID();

   // sort active tasks by time ascendently, inactive tasks by time descendently
   bool swapped=true;
   while (swapped) {
         swapped=false;
         for (int i=0;i<getNumObjects()-1;i++) {
             if ((!items[i]->isActive() && items[i+1]->isActive()) ||       // active/inactive
                 (items[i]->isActive()==items[i+1]->isActive() &&           // stejny typy
                    (items[i]->isActive()?
                       items[i]->getHash()>items[i+1]->getHash():
                       items[i]->getHash()<items[i+1]->getHash()
                    )
                 )
                ) {
                Action * temp=items[i];
                items[i]=items[i+1];
                items[i+1]=temp;
                swapped=true;
             }
         }
   }

   // najdu id a dam ho do index
   if (*index!=-1) {
      bool found=false;
      for (int i=0;!found && i<getNumObjects();i++)
          if (items[i]->getID()==id) {
             *index=i;
             found=true;
          }
   }
}