
#include "BBreminder.h"

PLUGIN_STRUCT *plugin;

struct plugindata {
   bool     autoDelete;
   char     format[512];
   int      leftMargin,topMargin,rightMargin,bottomMargin;
	struct {
	   StyleItem *list,*task,*select;
	} style;
   char     taskfile[MAX_PATH];
} plugindata;

#define  TASKS  6
const char *tasks[]={"popup window","run program","logoff","restart","suspend","shutdown"}; 

struct ICONS {
   HICON     icon[TASKS];
   int       size;
} icons;
  
int rowHeight;
int selected=-1,scroll=0;
int listsize_x,listsize_y,listpadsize_x,listpadsize_y;
int leftPadding=2,topPadding=1,rightPadding=2,bottomPadding=1;

ActionList  list;
UINT timer;

HBRUSH listBackground;


const char * BBpluginCallback_InfoQuery (int field)
{
	switch (field) {
          case PLUGIN_NAME:      return(szAppName);
          case PLUGIN_CLASSNAME: return(szClassName);
          case PLUGIN_VERSION:   return(szInfoVersion);  
          case PLUGIN_AUTHOR:    return(szInfoAuthor);   
          case PLUGIN_RELEASE:   return(szInfoRelDate); 
          case PLUGIN_LINK:      return(szInfoLink);   
          case PLUGIN_EMAIL:     return(szInfoEmail);   
          case PLUGIN_NAMEVER:   return(szVersion);
   }
   return(NULL);
}

bool BBpluginCallback_OnLoadPlugin()
{ 
   plugin=BBpluginExport_GetPlugin();

   sprintf(plugindata.taskfile,"%sBBreminder.tsx",plugin->directory);
   if (!TaskWindow_Create(plugin->hInstance,TASKS,tasks,onTaskChange) ||
       !PopupWindow_Create(plugin->hInstance)) return(false);

   timer=SetTimer(NULL,NULL,1000,(TIMERPROC)SynchronizeTimerProc);

   return(true);
}

void BBpluginCallback_OnUnloadPlugin()
{
   KillTimer(NULL,timer);
   if (listBackground) DeleteObject(listBackground);
   for (int i=0;i<TASKS;i++) if (icons.icon[i]) DestroyIcon(icons.icon[i]);
   TaskWindow_Destroy();
   PopupWindow_Destroy();
}

bool BBpluginCallback_OnMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg) {          
          case WM_KEYDOWN:
               switch (wParam) {
                      case VK_UP:    
                           if (GetAsyncKeyState(VK_CONTROL) & 0x8000) scroll--;
                           else selected--; 
                           break;
                      case VK_DOWN:  
                           if (GetAsyncKeyState(VK_CONTROL) & 0x8000) scroll++;
                           else selected++; 
                           break;
                      case VK_HOME:  selected=0; break;
                      case VK_END:   selected=list.getNumObjects()-1; break;
                      case VK_PRIOR: selected-=listpadsize_y/rowHeight; break;
                      case VK_NEXT : selected+=listpadsize_y/rowHeight; break;
                      case VK_RETURN: 
                           TaskWindow_Show(false);
                           if (selected!=-1) {
                              list.getObject(selected)->toggleActive();
                              onTaskChange();
                           }
                           return(true);
                      case VK_DELETE:
                           if (selected!=-1) {
                              list.removeObject(selected);
                              selected=-1;
                              while (scroll>list.getNumObjects()-1 && scroll>0) scroll--;
                              TaskWindow_Show(false);
                              onTaskChange();
                           }
                           return(true);
                      default: return(true);  // jina klavesa
               }
               if (selected<0) selected=0;
               if (selected>list.getNumObjects()-1) selected=list.getNumObjects()-1; 
               if (selected<scroll) scroll=selected;
               if ((selected-scroll+1)*rowHeight>listpadsize_y) scroll=selected-listpadsize_y/rowHeight+1;
               if (scroll<0) scroll=0;
               if (scroll>list.getNumObjects()-1) scroll=list.getNumObjects()-1;

               BBpluginExport_Repaint();
               return(true);

          case WM_LBUTTONDOWN: {
               selected=getClickPosition(LOWORD(lParam),HIWORD(lParam));
               if (selected!=-1) TaskWindow_SetAction(list.getObject(selected));
                  else TaskWindow_Show(false);
               BBpluginExport_Repaint();
               }
               break;

          case WM_LBUTTONDBLCLK:
               selected=getClickPosition(LOWORD(lParam),HIWORD(lParam));
               TaskWindow_Show(false);
               if (selected!=-1) {
                  list.getObject(selected)->toggleActive();
                  onTaskChange();
               }
               break;

          case WM_MOUSEWHEEL:
               if (GET_WHEEL_DELTA_WPARAM(wParam)>0) scroll--;
                  else scroll++;
               if (scroll<0) scroll=0;
               if (scroll>list.getNumObjects()-1) scroll=list.getNumObjects()-1;
               BBpluginExport_Repaint();
               return(true);

          case WM_WINDOWPOSCHANGED: 
               WINDOWPOS *wp; wp=(WINDOWPOS*)lParam;
               if (!(wp->flags & SWP_NOSIZE)) {
                  listsize_x=wp->cx-plugindata.leftMargin-plugindata.rightMargin;
                  listsize_y=wp->cy-plugindata.topMargin-plugindata.bottomMargin;
                  listpadsize_x=listsize_x-leftPadding-rightPadding;
                  listpadsize_y=listsize_y-topPadding-bottomPadding;
               }
               break;
   }
   return(false);
}

void BBpluginCallback_OnBroam (const char * broam)
{
   // "BBreminder.task." prefix
   if (BBplugin_IsBroam(&broam,TASK_PREFIX)) {
      if (BBplugin_IsBroam(&broam,EDIT_COMMAND)) {
         TaskWindow_Show(true);
      }                  
      if (BBplugin_IsBroam(&broam,ACTIVE_COMMAND)) {
         Action *action=list.getObject(selected);
         action->setActive(!action->isActive());
      }                  
      if (BBplugin_IsBroam(&broam,TASKTYPE_COMMAND)) {
         list.getObject(selected)->setType(atoi(broam));
      }                  
      if (BBplugin_IsBroam(&broam,TEXT_COMMAND)) {
         broam++;   // preskocit mezeru
         list.getObject(selected)->setText(broam);
      }                  
      if (BBplugin_IsBroam(&broam,HOUR_COMMAND)) {
         broam++;
         list.getObject(selected)->setHour(atoi(broam));
      }
      if (BBplugin_IsBroam(&broam,MINUTE_COMMAND)) {
         broam++;
         list.getObject(selected)->setMinute(atoi(broam));
      }
      if (BBplugin_IsBroam(&broam,DAY_COMMAND)) {
         broam++;
         list.getObject(selected)->setDay(atoi(broam));
      }
      if (BBplugin_IsBroam(&broam,MONTH_COMMAND)) {
         broam++;
         list.getObject(selected)->setMonth(atoi(broam));
      }
      if (BBplugin_IsBroam(&broam,YEAR_COMMAND)) {
         broam++;
         list.getObject(selected)->setYear(atoi(broam));
      }

      if (BBplugin_IsBroam(&broam,NEWTASK_COMMAND)) {
         SYSTEMTIME st;
         GetLocalTime(&st);
         selected=list.addObject(
            new Action(false,A_POPUP,
                       st.wHour,st.wMinute,st.wDay,st.wMonth,st.wYear,
                       "","",""));
         TaskWindow_SetAction(list.getObject(selected));
         TaskWindow_Show(true);
      }                  
      if (BBplugin_IsBroam(&broam,DELETETASK_COMMAND)) {
         list.removeObject(selected);
         selected=-1;
         while (scroll>list.getNumObjects()-1 && scroll>0) scroll--;
         TaskWindow_Show(false);
      }        
      if (BBplugin_IsBroam(&broam,DELALL_COMMAND)) {
         if (MessageBox(plugin->hwnd,"Do you really want to delete all tasks?","Question",
               MB_YESNO | MB_ICONQUESTION)==IDYES) {
            list.removeAll();
            selected=-1;
            scroll=0;
            TaskWindow_Show(false);
         }
      }

      onTaskChange();
   } // "BBreminder.task." prefix end
}
               
void BBpluginCallback_OnPaint (HDC hdc)
{
   RECT rect;

   HDC listDC=CreateCompatibleDC(hdc);
   HBITMAP listBitmap=CreateCompatibleBitmap(hdc,listsize_x,listsize_y);
   HGDIOBJ previousListBmp=SelectObject(listDC,listBitmap);

   SetRect(&rect,0,0,listsize_x,listsize_y);
   if (plugin->sctEnabled)
      FillRect(listDC,&rect,*BBpluginExport_GetSctBrush());
   else 
      if (plugindata.style.list) BBplugin_MakeStyleGradient(listDC,&rect,plugindata.style.list);
      else FillRect(listDC,&rect,listBackground);

   HGDIOBJ oldlistfont=SelectObject(listDC,BBpluginExport_GetBB()->font);
   SetBkMode(listDC,TRANSPARENT);

   for (int row=scroll;row<list.getNumObjects() && (row-scroll)*rowHeight<listpadsize_y;row++) {
       Action *action=list.getObject(row);
       //SetRect(&rect,1,1+(row-scroll)*icons.size,listsize_x-1,1+(row-scroll+1)*icons.size);
       SetRect(&rect,1,topPadding+(row-scroll)*rowHeight,
               listsize_x-1,topPadding+(row-scroll+1)*rowHeight);
       if (row==selected || !plugin->sctEnabled)
          BBplugin_MakeStyleGradient(listDC,&rect,(row==selected)?plugindata.style.select:plugindata.style.task);
       rect.left+=leftPadding;
       rect.right-=rightPadding;
       DrawState(listDC,0,NULL,(long)icons.icon[action->getType()],NULL,
                 rect.left,rect.top,0,0,DST_ICON|(action->isActive()?DSS_NORMAL:DSS_DISABLED));
       BBplugin_SetTextColor(listDC,(row==selected)?plugindata.style.select:plugindata.style.task);
       std::string text=formatOutput(action);
       if (icons.icon[action->getType()]!=NULL) rect.left+=icons.size+icons.size/6;
       DrawText(listDC,text.c_str(),text.size(),&rect,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
   }
/*   if (list.getNumObjects()==0) {
      BBplugin_SetTextColor(listDC,plugindata.style.list);
      SetRect(&rect,0,0,listsize_x,icons.size);
      if (plugin->sctEnabled) FillRect(listDC,&rect,listBackground);
      DrawText(listDC,"<no tasks>",10,&rect,DT_CENTER|DT_TOP);
   }*/

   BitBlt(hdc,
          plugindata.leftMargin,plugindata.topMargin,listsize_x,listsize_y,
          listDC,
          0,0,
          SRCCOPY);

   SelectObject(listDC,oldlistfont);   
   SelectObject(listDC,previousListBmp);
   DeleteObject(listBitmap);
   DeleteDC(listDC);
}

bool BBpluginCallback_OnCreateMenu (Menu * mainMenu)
{
   Menu *taskMenu,*typeMenu,*timeMenu;
   char broam[1000];
   const char *broam_prefix=BBpluginExport_GetBroamPrefix();

   mainMenu=MakeNamedMenu(szAppName,szAppName"_IDMain",true);
   sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,NEWTASK_COMMAND);
   MakeMenuItem(mainMenu,"New Task",broam,false);
   if (selected!=-1) {
      Action *action=list.getObject(selected);
      taskMenu=MakeNamedMenu("Task",szAppName"_IDTask",true);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,EDIT_COMMAND);
      MakeMenuItem(taskMenu,"Edit...",broam,false);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,ACTIVE_COMMAND);
      MakeMenuItem(taskMenu,"Active",broam,action->isActive());
      typeMenu=MakeNamedMenu("Type",szAppName"_IDType",true);
      short type=action->getType();
      for (int i=0;i<TASKS;i++) {
          sprintf(broam,"%s%s%s%d",broam_prefix,TASK_PREFIX,TASKTYPE_COMMAND,i);
          MakeMenuItem(typeMenu,tasks[i],broam,type==i);
      }
      MakeSubmenu(taskMenu,typeMenu,"Type");
      timeMenu=MakeNamedMenu("Time && Date",szAppName"_IDTime",true);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,HOUR_COMMAND);
      MakeMenuItemInt(timeMenu,"hour",broam,action->getHour(),0,23);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,MINUTE_COMMAND);
      MakeMenuItemInt(timeMenu,"minute",broam,action->getMinute(),0,59);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,DAY_COMMAND);
      MakeMenuItemInt(timeMenu,"day",broam,action->getDay(),0,31);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,MONTH_COMMAND);
      MakeMenuItemInt(timeMenu,"month",broam,action->getMonth(),0,12);
      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,YEAR_COMMAND);
      MakeMenuItemInt(timeMenu,"year",broam,action->getYear(),2000,2100);
      MakeSubmenu(taskMenu,timeMenu,"Time && Date");
/*      if (type==A_POPUP) 
         MakeMenuItemString(taskMenu,"Text",BROAM_PREFIX""TASK_PREFIX""TEXT_COMMAND,action->getText());
      if (type==A_RUN) {
      }*/
      MakeSubmenu(mainMenu,taskMenu,"Task");

      sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,DELETETASK_COMMAND);
      MakeMenuItem(mainMenu,"Delete Task",broam,false);
   }
   sprintf(broam,"%s%s%s",broam_prefix,TASK_PREFIX,DELALL_COMMAND);
   MakeMenuItem(mainMenu,"Delete All",broam,false);

   return(true);
}

void BBpluginCallback_OnConfigure (void)
{
   plugindata.leftMargin=BBplugin_ReadInt("margin.left",10);
   plugindata.topMargin=BBplugin_ReadInt("margin.top",10);
   plugindata.rightMargin=BBplugin_ReadInt("margin.right",10);
   plugindata.bottomMargin=BBplugin_ReadInt("margin.bottom",10);
   plugindata.autoDelete=BBplugin_ReadBool("autoDelete",false);
   strcpy(plugindata.format,BBplugin_ReadString("format","$t"));
   int tw_x=BBplugin_ReadInt("taskWindow.x",100);
   int tw_y=BBplugin_ReadInt("taskWindow.y",100);
   int pw_x=BBplugin_ReadInt("popupWindow.x",100);
   int pw_y=BBplugin_ReadInt("popupWindow.y",100);

   // getting style
   plugindata.style.list=(StyleItem *)GetSettingPtr(BBplugin_ParseStyle(BBplugin_ReadString("style.list","")));
   plugindata.style.task=(StyleItem *)GetSettingPtr(BBplugin_ParseStyle(BBplugin_ReadString("style.task","")));
   plugindata.style.select=(StyleItem *)GetSettingPtr(BBplugin_ParseStyle(BBplugin_ReadString("style.select","")));

   if (listBackground) DeleteObject(listBackground);
   if (plugindata.style.list==NULL) 
      listBackground=CreateSolidBrush(BBplugin_ReadColor("listBackgroundColor","#000000"));

   for (int i=0;i<TASKS;i++) if (icons.icon[i]) DestroyIcon(icons.icon[i]);
   icons.icon[0]=BBplugin_LoadIcon(BBplugin_ReadString("icons.popup",NULL));
   icons.icon[1]=BBplugin_LoadIcon(BBplugin_ReadString("icons.run",NULL));
   icons.icon[2]=BBplugin_LoadIcon(BBplugin_ReadString("icons.logoff",NULL));
   icons.icon[3]=BBplugin_LoadIcon(BBplugin_ReadString("icons.restart",NULL));
   icons.icon[4]=BBplugin_LoadIcon(BBplugin_ReadString("icons.suspend",NULL));
   icons.icon[5]=BBplugin_LoadIcon(BBplugin_ReadString("icons.shutdown",NULL));

   list.removeAll();
   loadTasks();

   LOGFONT logFont;
   ZeroMemory(&logFont,sizeof(LOGFONT));
   BB_STRUCT *BB=BBpluginExport_GetBB();
   GetObject(BB->font,sizeof(BB->font),&logFont);
   rowHeight=logFont.lfHeight+2;

//   icons.size=rowHeight;
   for (i=0;i<TASKS;i++)
       if (icons.icon[i]!=NULL) {
          ICONINFO poi;
          GetIconInfo(icons.icon[i],&poi);
          if (poi.xHotspot*2>icons.size) icons.size=poi.xHotspot*2;
          DeleteObject(poi.hbmMask);         // to je zrada!
          DeleteObject(poi.hbmColor);
       }
   if (icons.size>rowHeight) rowHeight=icons.size;

   TaskWindow_Configure(tw_x,tw_y,plugin->alwaysOnTop,plugin->windowStyleItem,plugindata.style.task,plugindata.style.select,BB->font,rowHeight);
   PopupWindow_Configure(pw_x,pw_y,plugin->windowStyleItem,plugindata.style.select,BB->font,rowHeight);
}

void BBpluginCallback_OnSaveConfiguration (void)
{
   BBplugin_WriteInt("taskWindow.x",TaskWindow_GetXposition());
   BBplugin_WriteInt("taskWindow.y",TaskWindow_GetYposition());
   BBplugin_WriteInt("popupWindow.x",PopupWindow_GetXposition());
   BBplugin_WriteInt("popupWindow.y",PopupWindow_GetYposition());
}


int getClickPosition (int mx, int my)
{
   if (mx>plugindata.leftMargin && my>plugindata.topMargin &&
       mx<plugin->width-plugindata.rightMargin && my<plugin->height-plugindata.bottomMargin) {
      bool clipped=false;
      for (int i=0;i<list.getNumObjects() && !clipped;i++)
          if (my>plugindata.topMargin+i*icons.size && my<plugindata.bottomMargin+(i+1)*icons.size) clipped=true;
      return(clipped?scroll+i-1:-1);   // _i_ pretece pri opusteni cyklu
   }
   return(-1);
}

std::string formatOutput (Action * action)
{
   std::string s=plugindata.format;
   replaceString(&s,"$h",toString(action->getHour()));
   replaceString(&s,"$i",toString(action->getMinute()));
   replaceString(&s,"$d",toString(action->getDay()));
   replaceString(&s,"$m",toString(action->getMonth()));
   replaceString(&s,"$y",toString(action->getYear()));
   replaceString(&s,"$t",action->getText());
   replaceString(&s,"$p",action->getProgram());
   replaceString(&s,"$r",action->getParams());
   replaceString(&s,"$a",action->isActive()?"active":"passive");
   return(s);
}

void onTaskChange (void)
{
   list.sort(&selected);
   saveTasks();
   BBpluginExport_Repaint();
}

// this sychronization sets application timer to second 0
TIMERPROC SynchronizeTimerProc (void)
{
   SYSTEMTIME st;
   GetLocalTime(&st);
   if (st.wSecond==0) {
      KillTimer(NULL,timer);
      timer=SetTimer(NULL,NULL,1000*60,(TIMERPROC)TaskTimerProc);
      TaskTimerProc();         // start even now
   }
   return(0);
}

TIMERPROC TaskTimerProc (void)
{
   SYSTEMTIME st;
   GetLocalTime(&st);

   for (int i=0;i<list.getNumObjects();i++) {
       Action *action=list.getObject(i);
       if (action->isActive() &&
           action->getHour()==st.wHour && action->getMinute()==st.wMinute &&
           action->getDay()==st.wDay && action->getMonth()==st.wMonth && action->getYear()==st.wYear) {
          switch (action->getType()) {
                 case A_POPUP: 
                      PopupWindow_SetText(action->getText());
                      PopupWindow_Show(true);
                      break;
                 case A_RUN:
                      ShellExecute(NULL,"open",action->getProgram(),action->getParams(),NULL,SW_SHOWNORMAL);
                      break;
                 case A_LOGOFF:
                      ExitWin(EWX_LOGOFF);
                      break;
                 case A_RESTART:
                      ExitWin(EWX_REBOOT);
                      break;
                 case A_SUSPEND:
                      SuspendWin();
                      break;
                 case A_SHUTDOWN:
                      ExitWin(EWX_POWEROFF);
                      break;
          }
          if (plugindata.autoDelete) list.removeObject(i);
             else action->setActive(false);
/*          list.sort(&selected);
          saveTasks();
          BBpluginExport_Repaint();*/
          onTaskChange();
       }
   }
  
   return(0);
}

void loadTasks (void)
{
   int num=ReadInt(plugindata.taskfile,szAppName".taskCount:",0);
   for (int i=0;i<num;i++) {
       const char *strID=szAppName".task";
       char str[100];
       sprintf(str,"%s[%d].active:",strID,i);
       bool active=ReadBool(plugindata.taskfile,str,false);
       sprintf(str,"%s[%d].type:",strID,i);
       short type=ReadInt(plugindata.taskfile,str,0);
       sprintf(str,"%s[%d].text:",strID,i);
       LPCSTR text=ReadString(plugindata.taskfile,str,"");
       sprintf(str,"%s[%d].minute:",strID,i);
       int minute=ReadInt(plugindata.taskfile,str,0);
       sprintf(str,"%s[%d].hour:",strID,i);
       int hour=ReadInt(plugindata.taskfile,str,0);
       sprintf(str,"%s[%d].day:",strID,i);
       int day=ReadInt(plugindata.taskfile,str,0);
       sprintf(str,"%s[%d].month:",strID,i);
       int month=ReadInt(plugindata.taskfile,str,0);
       sprintf(str,"%s[%d].year:",strID,i);
       int year=ReadInt(plugindata.taskfile,str,0);       
       sprintf(str,"%s[%d].program:",strID,i);
       LPCSTR program=ReadString(plugindata.taskfile,str,"");
       sprintf(str,"%s[%d].params:",strID,i);
       LPCSTR params=ReadString(plugindata.taskfile,str,"");
       list.addObject(new Action(active,type,hour,minute,day,month,year,text,program,params));
   }
}

void saveTasks (void)
{
   DeleteSetting(plugindata.taskfile,"*");
   WriteInt(plugindata.taskfile,szAppName".taskCount:",list.getNumObjects());
   for (int i=0;i<list.getNumObjects();i++) {
       Action *action=list.getObject(i);
       const char *strID=szAppName".task";
       char str[100];
       sprintf(str,"%s[%d].active:",strID,i);
       WriteBool(plugindata.taskfile,str,action->isActive());
       sprintf(str,"%s[%d].type:",strID,i);
       WriteInt(plugindata.taskfile,str,action->getType());
       sprintf(str,"%s[%d].text:",strID,i);
       WriteString(plugindata.taskfile,str,action->getText());
       sprintf(str,"%s[%d].minute:",strID,i);
       WriteInt(plugindata.taskfile,str,action->getMinute());
       sprintf(str,"%s[%d].hour:",strID,i);
       WriteInt(plugindata.taskfile,str,action->getHour());
       sprintf(str,"%s[%d].day:",strID,i);
       WriteInt(plugindata.taskfile,str,action->getDay());
       sprintf(str,"%s[%d].month:",strID,i);
       WriteInt(plugindata.taskfile,str,action->getMonth());
       sprintf(str,"%s[%d].year:",strID,i);
       WriteInt(plugindata.taskfile,str,action->getYear());
       sprintf(str,"%s[%d].program:",strID,i);
       WriteString(plugindata.taskfile,str,action->getProgram());
       sprintf(str,"%s[%d].params:",strID,i);
       WriteString(plugindata.taskfile,str,action->getParams());
   }
}
