
#include "PopupWindow.h"

struct pw {
   HWND      hwnd;
   HINSTANCE hInstance;
   HBITMAP   layout;
   int       x,y;
   HWND      okButton,textEdit;
   struct style {
      StyleItem *window,*select;
   } style;
   HFONT     font;
   int       rowHeight;
} pw;

HBRUSH pwEditBackground;


bool PopupWindow_Create (HINSTANCE hInstance)
{
   pw.hInstance=hInstance;

   WNDCLASS wc;
	ZeroMemory(&wc,sizeof(WNDCLASS));
	wc.lpfnWndProc=PopupWindow_WindowProc;      
	wc.hInstance=hInstance;    
	wc.lpszClassName=szPopupWindowClassName;    
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);

	if (!RegisterClass(&wc)) return(false);

   pw.hwnd=
      CreateWindowEx(
         WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
         szPopupWindowClassName,
         NULL,
         WS_POPUP,
         0,0,
         POPUPWINDOW_WIDTH,POPUPWINDOW_HEIGHT,
         NULL,
         NULL,
         hInstance,
         NULL);
   if (pw.hwnd==NULL) return(false);

   pw.textEdit=CreateWindowEx(0,TEXT("EDIT"),"",WS_CHILD|WS_VISIBLE|ES_MULTILINE|ES_CENTER|ES_AUTOVSCROLL|ES_READONLY,0,0,0,0,pw.hwnd,NULL,hInstance,NULL);
   pw.okButton=CreateWindowEx(0,TEXT("BUTTON"),"OK",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,0,0,0,0,pw.hwnd,NULL,hInstance,NULL);

   return(true);
}

void PopupWindow_Configure (int x, int y, StyleItem * windowStyle, StyleItem * selectStyle, HFONT font, int rowHeight)
{
   pw.x=x;
   pw.y=y;
   pw.style.window=windowStyle;
   pw.style.select=selectStyle;
   pw.font=font;
   pw.rowHeight=rowHeight;

   if (pwEditBackground) DeleteObject(pwEditBackground);
   pwEditBackground=CreateSolidBrush(pw.style.window?pw.style.window->Color:RGB(0,0,0));

   SetWindowPos(pw.hwnd,NULL,pw.x,pw.y,0,0,SWP_NOZORDER|SWP_NOSIZE);
   SetWindowPos(pw.textEdit,NULL,10,20,POPUPWINDOW_WIDTH-20,60,SWP_NOZORDER);
   SendMessage(pw.textEdit,WM_SETFONT,(WPARAM)pw.font,MAKEWORD(0,true));
   SetWindowPos(pw.okButton,NULL,POPUPWINDOW_WIDTH/2-25,85,50,rowHeight,SWP_NOZORDER);

   PopupWindow_Repaint();

/*   pw.textEdit=CreateWindowEx(0,TEXT("EDIT"),"",WS_CHILD|WS_VISIBLE|ES_MULTILINE|ES_CENTER|ES_AUTOVSCROLL|ES_READONLY,10,20,POPUPWINDOW_WIDTH-20,60,pw.hwnd,NULL,hInstance,NULL);
   SendMessage(pw.textEdit,WM_SETFONT,(WPARAM)pw.font,MAKEWORD(0,true));
   pw.okButton=CreateWindowEx(0,TEXT("BUTTON"),"OK",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,POPUPWINDOW_WIDTH/2-25,85,50,rowHeight,pw.hwnd,NULL,hInstance,NULL);*/
}

void PopupWindow_Destroy (void)
{
   if (pwEditBackground) DeleteObject(pwEditBackground);
   if (pw.layout) DeleteObject(pw.layout);
   DestroyWindow(pw.textEdit);
   DestroyWindow(pw.okButton);
   DestroyWindow(pw.hwnd);
   UnregisterClass(szPopupWindowClassName,pw.hInstance);
}

LRESULT CALLBACK PopupWindow_WindowProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   switch (message) {
          case WM_CREATE:
               MakeSticky(hwnd);
               break;
          case WM_PAINT:
               PopupWindow_Paint();
               break;

          case WM_COMMAND:
               if (HIWORD(wParam)==BN_CLICKED) 
                  if ((HWND)lParam==pw.okButton) 
                     ShowWindow(hwnd,SW_HIDE);
               break;
          case WM_LBUTTONDOWN: 
               if (HIWORD(lParam)<pw.rowHeight) {
                  PostMessage(hwnd,WM_SYSCOMMAND,0xF012,0);          // start moving
                  return(0);
               }
               break;

          case WM_WINDOWPOSCHANGED: 
               WINDOWPOS *wp; wp=(WINDOWPOS*)lParam;
               if (!(wp->flags & SWP_NOMOVE)) {
                  pw.x=wp->x;
                  pw.y=wp->y;        
               }
               break;

          case WM_DRAWITEM: 
               DRAWITEMSTRUCT *dis;dis=(DRAWITEMSTRUCT *)lParam;
               if (dis->CtlType==ODT_BUTTON) {
                  BBplugin_MakeStyleGradient(dis->hDC,&dis->rcItem,(dis->itemState & ODS_SELECTED)?pw.style.select:pw.style.window);
                  HGDIOBJ oldfont=SelectObject(dis->hDC,pw.font);
                  BBplugin_SetTextColor(dis->hDC,(dis->itemState & ODS_SELECTED)?pw.style.select:pw.style.window);
                  SetBkMode(dis->hDC,TRANSPARENT);
                  char text[20];
                  GetWindowText(dis->hwndItem,text,20);
                  DrawText(dis->hDC,text,strlen(text),&dis->rcItem,DT_CENTER|DT_VCENTER|DT_SINGLELINE);
                  SelectObject(dis->hDC,oldfont);
               }
               break;
          case WM_CTLCOLORSTATIC:
               BBplugin_SetTextColor((HDC)wParam,pw.style.window);
               SetBkMode((HDC)wParam,TRANSPARENT);
               return((BOOL)pwEditBackground); 

          default:
               return(DefWindowProc(hwnd,message,wParam,lParam));
   }
   return(0);
}

void PopupWindow_Paint (void)
{
   PAINTSTRUCT ps;
   HDC hdc=BeginPaint(pw.hwnd,&ps);  

   HDC myDC=CreateCompatibleDC(hdc);
	HGDIOBJ previousBmp;
   
   if (pw.layout==NULL) {
      pw.layout=CreateCompatibleBitmap(hdc,POPUPWINDOW_WIDTH,POPUPWINDOW_HEIGHT);
      previousBmp=SelectObject(myDC,pw.layout);

      RECT rect;
      SetRect(&rect,0,0,POPUPWINDOW_WIDTH,POPUPWINDOW_HEIGHT);
      BBplugin_MakeStyleGradient(myDC,&rect,pw.style.window);
      SetRect(&rect,0,0,POPUPWINDOW_WIDTH,pw.rowHeight);
      BBplugin_MakeStyleGradient(myDC,&rect,pw.style.select);
      HGDIOBJ oldfont=SelectObject(myDC,pw.font);
      SetBkMode(myDC,TRANSPARENT);
      BBplugin_SetTextColor(myDC,pw.style.select);
      DrawText(myDC,"BBreminder :: Popup Window",-1,&rect,DT_CENTER);
      SelectObject(myDC,oldfont);
   }
   else previousBmp=SelectObject(myDC,pw.layout);

	BitBlt(hdc,
	 		 ps.rcPaint.left,ps.rcPaint.top,ps.rcPaint.right-ps.rcPaint.left,ps.rcPaint.bottom-ps.rcPaint.top,
			 myDC,
			 ps.rcPaint.left,ps.rcPaint.top,
			 SRCCOPY);

   SelectObject(myDC,previousBmp);
	DeleteDC(myDC);

   EndPaint(pw.hwnd,&ps);
}

void PopupWindow_Repaint (void)
{
   if (pw.layout) {
      DeleteObject(pw.layout);
      pw.layout=NULL;
   }
   InvalidateRect(pw.hwnd,NULL,false);
}

void PopupWindow_Show (bool show)
{
   ShowWindow(pw.hwnd,show?SW_SHOW:SW_HIDE);
}

int PopupWindow_GetXposition (void)
{
   return(pw.x);
}

int PopupWindow_GetYposition (void)
{
   return(pw.y);
}

void PopupWindow_SetText (const char * text)
{
   SetWindowText(pw.textEdit,text);
}
