
#include "TaskWindow.h"

struct tw {
   HWND      hwnd;
   HINSTANCE hInstance;
   HBITMAP   layout;
   int       x,y;
   HWND      typeList,
             hourEdit,minuteEdit,dayEdit,monthEdit,yearEdit,
             programEdit,paramsEdit,textEdit,
             browseButton,updateButton,closeButton;
   int       numTask;
   struct style {
      StyleItem *window,*task,*select;
   } style;
   HFONT     font;
   int       rowHeight;
} tw;

HBRUSH twEditBackground;
Action *action;
const char ** twTasks;
TASKWINDOW_UPDATECALLBACK twUpdateCallback;

bool TaskWindow_Create (HINSTANCE hInstance, int numTask, const char ** tasks, TASKWINDOW_UPDATECALLBACK callback)
{
   tw.hInstance=hInstance;
   tw.numTask=numTask;
   twTasks=tasks;
   twUpdateCallback=callback;

   WNDCLASS wc;
	ZeroMemory(&wc,sizeof(WNDCLASS));
	wc.lpfnWndProc=TaskWindow_WindowProc;      
	wc.hInstance=hInstance;    
	wc.lpszClassName=szTaskWindowClassName;    
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);

	if (!RegisterClass(&wc)) return(false);

   tw.hwnd=
      CreateWindowEx(
         WS_EX_TOOLWINDOW,
         szTaskWindowClassName,
         NULL,
         WS_POPUP,
         0,0,
         TASKWINDOW_WIDTH,TASKWINDOW_HEIGHT,
         NULL,
         NULL,
         hInstance,
         NULL);
   if (tw.hwnd==NULL) return(false);

   tw.typeList=CreateWindowEx(0,TEXT("LISTBOX"),0,WS_CHILD|WS_VISIBLE|LBS_OWNERDRAWFIXED|LBS_HASSTRINGS,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.hourEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.minuteEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.dayEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.monthEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.yearEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.programEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.paramsEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.textEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
      
   tw.browseButton=CreateWindowEx(0,TEXT("BUTTON"),"...",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.updateButton=CreateWindowEx(0,TEXT("BUTTON"),"Update",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);
   tw.closeButton=CreateWindowEx(0,TEXT("BUTTON"),"Discard",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,0,0,0,0,tw.hwnd,NULL,hInstance,NULL);

   return(true);
}

void TaskWindow_Configure (int x, int y, bool alwaysOnTop, StyleItem * windowStyle, StyleItem * taskStyle, 
                           StyleItem * selectStyle, HFONT font, int rowHeight)
{
   tw.x=x;
   tw.y=y;
   tw.style.window=windowStyle;
   tw.style.task=taskStyle;
   tw.style.select=selectStyle;
   tw.font=font;
   tw.rowHeight=rowHeight;

   if (twEditBackground) DeleteObject(twEditBackground);
   twEditBackground=CreateSolidBrush(tw.style.window?tw.style.window->Color:RGB(0,0,0));

   SetWindowPos(tw.hwnd,alwaysOnTop?HWND_TOPMOST:HWND_NOTOPMOST,tw.x,tw.y,0,0,SWP_NOSIZE);
   SetWindowPos(tw.typeList,NULL,5,20+tw.rowHeight,100,160,SWP_NOZORDER);
   SendMessage(tw.typeList,LB_RESETCONTENT,0,0);
   for (int i=0;i<tw.numTask;i++)
       SendMessage(tw.typeList,LB_ADDSTRING,0,(LPARAM)twTasks[i]);
   SendMessage(tw.typeList,LB_SETCURSEL,0,0);
   SetWindowPos(tw.hourEdit,NULL,120,20+tw.rowHeight,30,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.hourEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.minuteEdit,NULL,160,20+tw.rowHeight,30,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.minuteEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.dayEdit,NULL,120,20+tw.rowHeight*3,30,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.dayEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.monthEdit,NULL,160,20+tw.rowHeight*3,30,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.monthEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.yearEdit,NULL,200,20+tw.rowHeight*3,30,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.yearEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.programEdit,NULL,120,20+tw.rowHeight*6,90,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.programEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.paramsEdit,NULL,120,20+tw.rowHeight*8,110,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.paramsEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   SetWindowPos(tw.textEdit,NULL,120,20+tw.rowHeight*11,110,tw.rowHeight,SWP_NOZORDER);
   SendMessage(tw.textEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));

   SetWindowPos(tw.browseButton,NULL,210,20+tw.rowHeight*6,20,tw.rowHeight,SWP_NOZORDER);
   SetWindowPos(tw.updateButton,NULL,5,210,50,tw.rowHeight,SWP_NOZORDER);
   SetWindowPos(tw.closeButton,NULL,60,210,50,tw.rowHeight,SWP_NOZORDER);

   TaskWindow_Repaint();

/*   tw.typeList=CreateWindowEx(0,TEXT("LISTBOX"),0,WS_CHILD|WS_VISIBLE|LBS_OWNERDRAWFIXED|LBS_HASSTRINGS,5,20+tw.rowHeight,100,160,tw.hwnd,NULL,hInstance,NULL);
   tw.hourEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,120,20+tw.rowHeight,30,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.hourEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.minuteEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,160,20+tw.rowHeight,30,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.minuteEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.dayEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,120,20+tw.rowHeight*3,30,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.dayEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.monthEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,160,20+tw.rowHeight*3,30,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.monthEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.yearEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_NUMBER|ES_AUTOHSCROLL,200,20+tw.rowHeight*3,30,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.yearEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.programEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL,120,20+tw.rowHeight*6,90,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.programEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.paramsEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL,120,20+tw.rowHeight*8,110,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.paramsEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
   tw.textEdit=CreateWindowEx(0,TEXT("EDIT"),"nic",WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL,120,20+tw.rowHeight*11,110,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   SendMessage(tw.textEdit,WM_SETFONT,(WPARAM)font,MAKEWORD(0,true));
      
   for (int i=0;i<tw.numTask;i++)
       SendMessage(tw.typeList,LB_ADDSTRING,0,(LPARAM)tasks[i]);
   SendMessage(tw.typeList,LB_SETCURSEL,0,0);

   tw.browseButton=CreateWindowEx(0,TEXT("BUTTON"),"...",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,210,20+tw.rowHeight*6,20,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   tw.updateButton=CreateWindowEx(0,TEXT("BUTTON"),"Update",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,5,210,50,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
   tw.closeButton=CreateWindowEx(0,TEXT("BUTTON"),"Discard",WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,60,210,50,tw.rowHeight,tw.hwnd,NULL,hInstance,NULL);
*/

}

void TaskWindow_Destroy (void)
{
   if (twEditBackground) DeleteObject(twEditBackground);
   if (tw.layout) DeleteObject(tw.layout);
   DestroyWindow(tw.typeList);
   DestroyWindow(tw.hourEdit);
   DestroyWindow(tw.dayEdit);
   DestroyWindow(tw.monthEdit);
   DestroyWindow(tw.yearEdit);
   DestroyWindow(tw.programEdit);
   DestroyWindow(tw.paramsEdit);
   DestroyWindow(tw.textEdit);
   DestroyWindow(tw.browseButton);
   DestroyWindow(tw.updateButton);
   DestroyWindow(tw.closeButton);
   DestroyWindow(tw.hwnd);
   UnregisterClass(szTaskWindowClassName,tw.hInstance);
}

LRESULT CALLBACK TaskWindow_WindowProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   switch (message) {
          case WM_CREATE:
               tw.layout=NULL;
               MakeSticky(hwnd);
               break;
          case WM_PAINT:
               TaskWindow_Paint();
               break;

          case WM_COMMAND:
               if (HIWORD(wParam)==BN_CLICKED) {
                  if ((HWND)lParam==tw.browseButton) {
                     OPENFILENAME of;
                     ZeroMemory(&of,sizeof(OPENFILENAME));
                     of.lStructSize=sizeof(OPENFILENAME);
                     of.hwndOwner=tw.hwnd;
                     char file[1024]={"\0"};         // 1st character must be zero
                     of.lpstrFile=file;
                     of.nMaxFile=1024;
                     of.Flags=OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
                     of.lpstrFilter="All Files (*.*)\0*.*\0";
                     if (GetOpenFileName(&of))
                        SetWindowText(tw.programEdit,file);
                  }
                  if ((HWND)lParam==tw.updateButton) { 
                     grabTaskWindow();
                     twUpdateCallback();
                     ShowWindow(hwnd,SW_HIDE);
                  }
                  if ((HWND)lParam==tw.closeButton) {
                     ShowWindow(hwnd,SW_HIDE);
                  }
               }
               break;
          case WM_LBUTTONDOWN: 
               if (HIWORD(lParam)<tw.rowHeight) {
                  PostMessage(hwnd,WM_SYSCOMMAND,0xF012,0);          // start moving
                  return(0);
               }
               break;
          case WM_CHAR:
               if (wParam==27) ShowWindow(hwnd,SW_HIDE);
               return(0);

          case WM_WINDOWPOSCHANGED: 
               WINDOWPOS *wp; wp=(WINDOWPOS*)lParam;
               if (!(wp->flags & SWP_NOMOVE)) {
                  tw.x=wp->x;
                  tw.y=wp->y;        
               }
               break;

          case WM_MEASUREITEM:
               MEASUREITEMSTRUCT *mis;mis=(MEASUREITEMSTRUCT *)lParam;
               if (mis->CtlType==ODT_LISTBOX) {
                  mis->itemWidth=50;
                  mis->itemHeight=tw.rowHeight;
                  return(true);
               }
               break;
          case WM_DRAWITEM: 
               DRAWITEMSTRUCT *dis;dis=(DRAWITEMSTRUCT *)lParam;
               if (dis->CtlType==ODT_LISTBOX || dis->CtlType==ODT_BUTTON) {
                  HGDIOBJ oldfont=SelectObject(dis->hDC,tw.font);
                  SetBkMode(dis->hDC,TRANSPARENT);
                  char text[20];
                  if (dis->CtlType==ODT_LISTBOX) {
                     BBplugin_MakeStyleGradient(dis->hDC,&dis->rcItem,(dis->itemState & ODS_SELECTED)?tw.style.select:tw.style.task);
                     BBplugin_SetTextColor(dis->hDC,(dis->itemState & ODS_SELECTED)?tw.style.select:tw.style.task);
                     SendMessage(tw.typeList,LB_GETTEXT,dis->itemID,(LPARAM)(char*)text);
                     TextOut(dis->hDC,5,dis->rcItem.top,text,strlen(text));
                  }
                  else {
                     BBplugin_MakeStyleGradient(dis->hDC,&dis->rcItem,(dis->itemState & ODS_SELECTED)?tw.style.select:tw.style.window);
                     BBplugin_SetTextColor(dis->hDC,(dis->itemState & ODS_SELECTED)?tw.style.select:tw.style.window);
                     GetWindowText(dis->hwndItem,text,20);
                     DrawText(dis->hDC,text,strlen(text),&dis->rcItem,DT_CENTER|DT_VCENTER|DT_SINGLELINE);
                  }
                  SelectObject(dis->hDC,oldfont);
               }
               break;
          case WM_CTLCOLORLISTBOX:
          case WM_CTLCOLOREDIT:
               BBplugin_SetTextColor((HDC)wParam,tw.style.window);
               SetBkMode((HDC)wParam,TRANSPARENT);
               return((BOOL)twEditBackground); 

          default:
               return(DefWindowProc(hwnd,message,wParam,lParam));
   }
   return(0);
}

void TaskWindow_Paint (void)
{
   PAINTSTRUCT ps;
   HDC hdc=BeginPaint(tw.hwnd,&ps);  

   HDC myDC=CreateCompatibleDC(hdc);
	HGDIOBJ previousBmp;
   
   if (tw.layout==NULL) {
      tw.layout=CreateCompatibleBitmap(hdc,TASKWINDOW_WIDTH,TASKWINDOW_HEIGHT);
      previousBmp=SelectObject(myDC,tw.layout);

      RECT rect;
      SetRect(&rect,0,0,TASKWINDOW_WIDTH,TASKWINDOW_HEIGHT);
      BBplugin_MakeStyleGradient(myDC,&rect,tw.style.window);
      SetRect(&rect,0,0,TASKWINDOW_WIDTH,tw.rowHeight);
      BBplugin_MakeStyleGradient(myDC,&rect,tw.style.select);
      HGDIOBJ oldfont=SelectObject(myDC,tw.font);
      SetBkMode(myDC,TRANSPARENT);
      BBplugin_SetTextColor(myDC,tw.style.select);
      DrawText(myDC,"BBreminder :: Task Definition",-1,&rect,DT_CENTER);

      BBplugin_SetTextColor(myDC,tw.style.window);
      TextOut(myDC,5,20,"Task Type",9);
      TextOut(myDC,120,20,"Hour",4);
      TextOut(myDC,160,20,"Minute",6);
      TextOut(myDC,120,20+tw.rowHeight*2,"Day",3);
      TextOut(myDC,160,20+tw.rowHeight*2,"Month",5);
      TextOut(myDC,200,20+tw.rowHeight*2,"Year",4);
      TextOut(myDC,120,20+tw.rowHeight*5,"Program",7);
      TextOut(myDC,120,20+tw.rowHeight*7,"Params",6);
      TextOut(myDC,120,20+tw.rowHeight*10,"Text",4);

      SelectObject(myDC,oldfont);
   }
   else previousBmp=SelectObject(myDC,tw.layout);

	BitBlt(hdc,
	 		 ps.rcPaint.left,ps.rcPaint.top,ps.rcPaint.right-ps.rcPaint.left,ps.rcPaint.bottom-ps.rcPaint.top,
			 myDC,
			 ps.rcPaint.left,ps.rcPaint.top,
			 SRCCOPY);

   SelectObject(myDC,previousBmp);
	DeleteDC(myDC);

   EndPaint(tw.hwnd,&ps);
}

void TaskWindow_Repaint (void)
{
   if (tw.layout) {
      DeleteObject(tw.layout);
      tw.layout=NULL;
   }
   InvalidateRect(tw.hwnd,NULL,false);
}

void TaskWindow_Show (bool show)
{
   ShowWindow(tw.hwnd,show?SW_SHOW:SW_HIDE);
}
   
void TaskWindow_SetAction (Action * a)
{
   action=a;

   char retz[255];
   SendMessage(tw.typeList,LB_SETCURSEL,action->getType(),0);
   itoa(action->getHour(),retz,10);
   SetWindowText(tw.hourEdit,retz);
   itoa(action->getMinute(),retz,10);
   SetWindowText(tw.minuteEdit,retz);
   itoa(action->getDay(),retz,10);
   SetWindowText(tw.dayEdit,retz);
   itoa(action->getMonth(),retz,10);
   SetWindowText(tw.monthEdit,retz);
   itoa(action->getYear(),retz,10);
   SetWindowText(tw.yearEdit,retz);
   SetWindowText(tw.programEdit,action->getProgram());
   SetWindowText(tw.paramsEdit,action->getParams());
   SetWindowText(tw.textEdit,action->getText());
}

void grabTaskWindow (void)
{
   char retz[1024];
   action->setType(SendMessage(tw.typeList,LB_GETCURSEL,0,0));
   GetWindowText(tw.hourEdit,retz,10);
   action->setHour(atoi(retz));
   GetWindowText(tw.minuteEdit,retz,10);
   action->setMinute(atoi(retz));
   GetWindowText(tw.dayEdit,retz,10);
   action->setDay(atoi(retz));
   GetWindowText(tw.monthEdit,retz,10);
   action->setMonth(atoi(retz));
   GetWindowText(tw.yearEdit,retz,10);
   action->setYear(atoi(retz));
   GetWindowText(tw.programEdit,retz,1024);
   action->setProgram(retz);
   GetWindowText(tw.paramsEdit,retz,1024);
   action->setParams(retz);
   GetWindowText(tw.textEdit,retz,1024);
   action->setText(retz);
}

int TaskWindow_GetXposition (void)
{
   return(tw.x);
}

int TaskWindow_GetYposition (void)
{
   return(tw.y);
}