
#include "BBrssfeed.h"

PLUGIN_STRUCT *plugin;

struct plugindata {
   int      timer;
   int      scrollSpeed;
   int      leftMargin,topMargin,rightMargin,bottomMargin;
   bool     showTitlebar;
   bool     popupNotify;
   int      descLines;
   bool     timerUpdate,reconfUpdate;
	struct {
	   StyleItem *list,*select,*description;
	} style;
} plugindata;


HPEN pen;
UINT timer;//,hidetimer;
//bool hidden=false;
//RECT hiderect;
int rowHeight,itemHeight,scroll=0,selected=-1;
int listsize_x,listsize_y,listpadsize_x,listpadsize_y;
//bool no_paint=false;
int  leftPadding=5,topPadding=2,rightPadding=5,bottomPadding=2;
HANDLE thandle;
bool thread_running=false;
HttpResponse * connection;

typedef struct {
   const char *text, *url, *description;
   bool header;
} LISTITEM;
std::vector<LISTITEM> listitems;

RSSReader rssreader;


const char * BBpluginCallback_InfoQuery (int field)
{
	switch (field) {
          case PLUGIN_NAME:      return(szAppName);
          case PLUGIN_CLASSNAME: return(szClassName);
          case PLUGIN_VERSION:   return(szInfoVersion);  
          case PLUGIN_AUTHOR:    return(szInfoAuthor);   
          case PLUGIN_RELEASE:   return(szInfoRelDate); 
          case PLUGIN_LINK:      return(szInfoLink);   
          case PLUGIN_EMAIL:     return(szInfoEmail);   
          case PLUGIN_NAMEVER:   return(szVersion);
   }
   return(NULL);
}

bool BBpluginCallback_OnLoadPlugin()
{ 
   plugin=BBpluginExport_GetPlugin();

   return(true);
}

void BBpluginCallback_OnUnloadPlugin()
{
   KillTimer(NULL,timer);
   if (thread_running) {
      connection->shutDown();
      // wait for thread to terminate
      unsigned long ec;
      do {
         GetExitCodeThread(thandle,&ec);
         Sleep(10);
      } while (ec==STILL_ACTIVE);
   }
   if (pen) DeleteObject(pen);
}

bool BBpluginCallback_OnMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg) {          
          case WM_KEYDOWN:
               switch (wParam) {
                      case VK_UP:    
                           if (GetAsyncKeyState(VK_CONTROL) & 0x8000) scroll--;
                           else selected--; 
                           break;
                      case VK_DOWN:  
                           if (GetAsyncKeyState(VK_CONTROL) & 0x8000) scroll++;
                           else selected++; 
                           break;
                      case VK_HOME:  selected=0; break;
                      case VK_END:   selected=listitems.size()-1; break;
                      case VK_PRIOR: selected-=listpadsize_y/itemHeight; break;
                      case VK_NEXT : selected+=listpadsize_y/itemHeight; break;
                      case VK_RETURN: 
                         if (selected!=-1) 
                            ShellExecute(hwnd,"open",listitems[selected].url,NULL,NULL,SW_SHOW);
                         break;
                      default: return(true);  // jina klavesa
               }
               if (selected<0) selected=0;
               if (selected>listitems.size()-1) selected=listitems.size()-1; 
               if (selected<scroll) scroll=selected;
               if ((selected-scroll+1)*itemHeight>listpadsize_y) scroll=selected-listpadsize_y/itemHeight+1;
               if (scroll<0) scroll=0;
               if (scroll>listitems.size()-1) scroll=listitems.size()-1;

               BBpluginExport_Repaint();
               return(true);

          case WM_LBUTTONDOWN:
               selected=getClickPosition(LOWORD(lParam),HIWORD(lParam));
               BBpluginExport_Repaint();
               break;

          case WM_LBUTTONDBLCLK:
               if (selected!=-1) 
                  ShellExecute(hwnd,"open",listitems[selected].url,NULL,NULL,SW_SHOW);
               return(true);

          case WM_MOUSEWHEEL:
               if (GET_WHEEL_DELTA_WPARAM(wParam)>0) scroll-=plugindata.scrollSpeed;
                  else scroll+=plugindata.scrollSpeed;
               if (scroll<0) scroll=0;
               if (scroll>listitems.size()-1) scroll=listitems.size()-1;
               BBpluginExport_Repaint();
               return(true);

          case WM_WINDOWPOSCHANGED: {
               WINDOWPOS *wp=(WINDOWPOS*)lParam;
               if (!(wp->flags & SWP_NOSIZE)) {
                  listsize_x=wp->cx-plugindata.leftMargin-plugindata.rightMargin;
                  listsize_y=wp->cy-plugindata.topMargin-plugindata.bottomMargin;
                  listpadsize_x=listsize_x-leftPadding-rightPadding;
                  listpadsize_y=listsize_y-topPadding-bottomPadding;
               }
               break;
          }
   }

   return(false);
}

void BBpluginCallback_OnBroam (const char * broam)
{
   //MessageBox(0,broam,0,0);
   if (BBplugin_IsBroam(&broam,TIMERUPDATE_COMMAND)) {
      plugindata.timerUpdate=!plugindata.timerUpdate;
   }
   if (BBplugin_IsBroam(&broam,FORCEUPDATE_COMMAND)) {
      StartDownloadThread();
   }
   if (BBplugin_IsBroam(&broam,FEEDTOGGLE_COMMAND)) {
      // zbyva jenom cislo feedu
      int feed=atoi(broam);
      rssreader.getFeed(feed)->toggleEnabled();
      updateList();
   }
   if (BBplugin_IsBroam(&broam,FEEDMAXITEMS_COMMAND)) {
      broam++;
      int f,n;
      sscanf(broam,"%d %d",&f,&n);
      rssreader.getFeed(f)->setMaxItems(n);
      updateList();
   }
   if (BBplugin_IsBroam(&broam,FEEDCAPTION_COMMAND)) {
      broam++;
      int f=atoi(broam);
      const char *p=strchr(broam,' ')+1;
      if (p) rssreader.getFeed(f)->setCaption(p);
   }
   if (BBplugin_IsBroam(&broam,FEEDURL_COMMAND)) {
      broam++;
      int f=atoi(broam);
      const char *p=strchr(broam,' ')+1;
      if (p) rssreader.getFeed(f)->setUrl(p);
   }
}

void BBpluginCallback_OnPaint (HDC hdc)
{
   RECT rect;

   if (plugindata.showTitlebar) {
      SetRect(&rect,0,0,plugin->width,rowHeight);
      BBplugin_MakeStyleGradient(hdc,&rect,plugindata.style.select);
      SetBkMode(hdc,TRANSPARENT);
      BBplugin_SetTextColor(hdc,plugindata.style.select);
      rect.left=5;
      DrawText(hdc,szAppName,-1,&rect,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
   }

   // hdc a bitmapa pro list
   HDC listDC=CreateCompatibleDC(hdc);
   HBITMAP listBitmap=CreateCompatibleBitmap(hdc,listsize_x,listsize_y);
   HGDIOBJ previousListBmp=SelectObject(listDC,listBitmap);

   SetRect(&rect,0,0,listsize_x,listsize_y);
   if (!plugin->sctEnabled) 
      BBplugin_MakeStyleGradient(listDC,&rect,plugindata.style.list);
	else 
	   FillRect(listDC,&rect,*BBpluginExport_GetSctBrush());

   HGDIOBJ oldlistfont=SelectObject(listDC,BBpluginExport_GetBB()->font);
   SetBkMode(listDC,TRANSPARENT);
   HGDIOBJ oldpen=SelectObject(listDC,pen);

   for (int row=scroll;row<listitems.size() && (row-scroll)*itemHeight<listpadsize_y;row++) {
       SetRect(&rect,1,topPadding+(row-scroll)*itemHeight,
               listsize_x-1,topPadding+(row-scroll+1)*itemHeight+2);
       if (row==selected)
          BBplugin_MakeStyleGradient(listDC,&rect,plugindata.style.select);
       rect.left+=leftPadding;
       rect.right-=rightPadding;
       BBplugin_SetTextColor(listDC,(row==selected)?plugindata.style.select:plugindata.style.list);
       DrawText(listDC,listitems[row].text,strlen(listitems[row].text),&rect,DT_LEFT|DT_TOP|DT_SINGLELINE);
       if (listitems[row].header) {
          MoveToEx(listDC,leftPadding,topPadding+(row-scroll+1)*itemHeight,NULL);
          LineTo(listDC,leftPadding+listpadsize_x,topPadding+(row-scroll+1)*itemHeight);
       }
       rect.top+=rowHeight;
       //rect.bottom+=(rowHeight*(plugindata.descLines));
       rect.left+=20;
       //rect.right=rect.left+100;
       //MakeStyleGradient(listDC,&rect,plugindata.style.select,true);
       BBplugin_SetTextColor(listDC,(row==selected)?plugindata.style.select:plugindata.style.description);
       DrawText(listDC,listitems[row].description,-1,&rect,DT_LEFT|DT_TOP|DT_WORDBREAK/*|DT_WORD_ELLIPSIS|DT_MODIFYSTRING*/);
   }

   BitBlt(hdc,
          plugindata.leftMargin,plugindata.topMargin,
          listsize_x,listsize_y,
          listDC,
          0,0,
          SRCCOPY);

   SelectObject(listDC,oldpen);
   SelectObject(listDC,oldlistfont);   
   SelectObject(listDC,previousListBmp);
   DeleteObject(listBitmap);
   DeleteDC(listDC);
}

bool BBpluginCallback_OnCreateMenu (Menu * mainMenu)
{
   Menu *feedsMenu;
   char broam[1000];
   const char *broam_prefix=BBpluginExport_GetBroamPrefix();

   feedsMenu=MakeNamedMenu("Feeds",szAppName"_IDFeeds",true);
   for (int f=0;f<rssreader.getNumFeeds();f++) {
       Menu *feedmenu;
       RSSFeed * rssfeed=rssreader.getFeed(f);
       char id[1000];
       sprintf(id,"%s_IDFeed%d",szAppName,f);
       feedmenu=MakeNamedMenu(rssfeed->getCaption(),id,true);

       sprintf(broam,"%s%s %d",broam_prefix,FEEDTOGGLE_COMMAND,f);
       MakeMenuItem(feedmenu,"enabled",broam,rssfeed->isEnabled());
       sprintf(broam,"%s%s %d",broam_prefix,FEEDCAPTION_COMMAND,f);
       MakeMenuItemString(feedmenu,"caption",broam,rssfeed->getCaption());
       sprintf(broam,"%s%s %d",broam_prefix,FEEDURL_COMMAND,f);
       MakeMenuItemString(feedmenu,"url",broam,rssfeed->getUrl());
       sprintf(broam,"%s%s %d",broam_prefix,FEEDMAXITEMS_COMMAND,f);
       MakeMenuItemInt(feedmenu,"max items",broam,rssfeed->getMaxItems(),0,100);

       //MakeMenuItem(feedsMenu,rssfeed->getCaption(),broam,rssfeed->isEnabled());
       MakeSubmenu(feedsMenu,feedmenu,rssfeed->getCaption());
   }

   MakeSubmenu(mainMenu,feedsMenu,"Feeds");
   MakeMenuNOP(mainMenu,NULL);
   sprintf(broam,"%s%s",broam_prefix,TIMERUPDATE_COMMAND);
   MakeMenuItem(mainMenu,"Timer Update",broam,plugindata.timerUpdate);
   sprintf(broam,"%s%s",broam_prefix,FORCEUPDATE_COMMAND);
   MakeMenuItem(mainMenu,"Force Update!",broam,false);

   return(true);
}

void BBpluginCallback_OnConfigure (void)
{
   rssreader.clear();

   plugindata.timer=BBplugin_ReadInt("timer",10);
   plugindata.showTitlebar=BBplugin_ReadBool("showTitlebar",false);
   plugindata.scrollSpeed=BBplugin_ReadInt("scrollSpeed",1);
   plugindata.popupNotify=BBplugin_ReadBool("popupNotify",false);
   plugindata.descLines=BBplugin_ReadInt("descriptionLines",0);
   plugindata.leftMargin=BBplugin_ReadInt("margin.left",10);
   plugindata.topMargin=BBplugin_ReadInt("margin.top",10);
   plugindata.rightMargin=BBplugin_ReadInt("margin.right",10);
   plugindata.bottomMargin=BBplugin_ReadInt("margin.bottom",10);
   plugindata.timerUpdate=BBplugin_ReadBool("timerUpdate",false);
   plugindata.reconfUpdate=BBplugin_ReadBool("reconfigureUpdate",false);
   
   int feeds=BBplugin_ReadInt("feedCount",0);
   for (int i=1;i<=feeds;i++) {
       char str1[100],str2[100],str3[100];
       sprintf(str1,"feed[%d].caption",i);
       sprintf(str2,"feed[%d].url",i);
       sprintf(str3,"feed[%d].enabled",i);
       RSSFeed *rssfeed=
          new RSSFeed(
                 BBplugin_ReadString(str1,""),
                 BBplugin_ReadString(str2,""),
                 BBplugin_ReadBool(str3,false)
          );
       rssreader.addFeed(rssfeed);
       sprintf(str1,"feed[%d].maxItems",i);
       rssfeed->setMaxItems(BBplugin_ReadInt(str1,20));
   }

   // getting style
   plugindata.style.list=(StyleItem *)GetSettingPtr(BBplugin_ParseStyle(BBplugin_ReadString("style.list","")));
   plugindata.style.select=(StyleItem *)GetSettingPtr(BBplugin_ParseStyle(BBplugin_ReadString("style.select","")));
   plugindata.style.description=(StyleItem *)GetSettingPtr(BBplugin_ParseStyle(BBplugin_ReadString("style.description","")));

   LOGFONT logFont;
   ZeroMemory(&logFont,sizeof(LOGFONT));
   BB_STRUCT *BB=BBpluginExport_GetBB();
   GetObject(BB->font,sizeof(BB->font),&logFont);
   rowHeight=logFont.lfHeight;//+2;
   itemHeight=rowHeight*(plugindata.descLines+1);

   if (pen) DeleteObject(pen);
   pen=CreatePen(PS_SOLID,1,plugindata.style.list?plugindata.style.list->TextColor:RGB(0,0,0));

   if (timer) KillTimer(NULL,timer);
   timer=SetTimer(NULL,NULL,plugindata.timer*1000*60,(TIMERPROC)TimerProc);

   updateList();

   if (plugindata.reconfUpdate) StartDownloadThread();
}

void BBpluginCallback_OnSaveConfiguration (void)
{
   BBplugin_WriteBool("timerUpdate",plugindata.timerUpdate);
   for (int f=0;f<rssreader.getNumFeeds();f++) {
       char str[1000];
       sprintf(str,"feed[%d].enabled",f+1);
       BBplugin_WriteBool(str,rssreader.getFeed(f)->isEnabled());
       sprintf(str,"feed[%d].caption",f+1);
       BBplugin_WriteString(str,rssreader.getFeed(f)->getCaption());
       sprintf(str,"feed[%d].url",f+1);
       BBplugin_WriteString(str,rssreader.getFeed(f)->getUrl());
       sprintf(str,"feed[%d].maxItems",f+1);
       BBplugin_WriteInt(str,rssreader.getFeed(f)->getMaxItems());
   }
}


void updateList (void)
{
   //no_paint=true;
   listitems.clear();
   for (int f=0;f<rssreader.getNumFeeds();f++) {
       RSSFeed * rssfeed=rssreader.getFeed(f);
       if (rssfeed->isEnabled()) {
          LISTITEM item;
          item.text=rssfeed->getTitle();
          if (!strlen(item.text)) item.text=rssfeed->getCaption();
          item.url=rssfeed->getLink();
          item.description=rssfeed->getDescription();
          item.header=true;
          listitems.insert(listitems.end(),item);
          if (rssfeed->getNumItems()) 
             for (int i=0;i<rssfeed->getNumItems() && i<rssfeed->getMaxItems();i++) {
                 RSSItem *rssitem=rssfeed->getItem(i);
                 LISTITEM item;              
                 item.text=rssitem->getTitle();
                 item.url=rssitem->getLink();
                 item.description=rssitem->getDescription();
                 item.header=false;
                 listitems.insert(listitems.end(),item);       
             }
       }
   }
   //no_paint=false;
   BBpluginExport_Repaint();
}

int getClickPosition (int mx, int my)
{
   if (mx>plugindata.leftMargin && my>plugindata.topMargin+topPadding && 
       mx<plugin->width-plugindata.rightMargin && my<plugin->height-plugindata.bottomMargin-bottomPadding) {
      int n=(my-plugindata.topMargin-topPadding)/itemHeight+scroll;
      return(n<listitems.size()?n:-1);
   }
   return(-1);
}

TIMERPROC TimerProc (void)
{
   if (plugindata.timerUpdate) StartDownloadThread();
   return(0);
}

void StartDownloadThread (void)
{
   if (!thread_running) {
      thandle=CreateThread(NULL,0,DownloadThread,0,0,0);
   }
}
   
DWORD WINAPI DownloadThread (LPVOID)
{
   thread_running=true;
   HttpResponse response;
   connection=&response;

   for (int f=0;f<rssreader.getNumFeeds();f++) {
       RSSFeed * rssfeed=rssreader.getFeed(f);
       if (!rssfeed->isEnabled()) continue;
       RSSFeed oldfeed(rssfeed);
       rssfeed->clear();   
       updateList();         // je treba zneplatnit neplatne linky v listitems!

       int respCode=response.grab_www(rssfeed->getUrl(),"User-Agent: BBrssfeed@BB4Win\r\n");
       if (respCode!=RESPONSE_OK) {
          rssfeed->setError(" - ERROR: Unable to download data from server");
          updateList();
          if (respCode==RESPONSE_SHUTDOWN) {
             thread_running=false;
             return(1);
          }
          continue;
       }
       //MessageBox(0,(const char *)response.getContentData(),rssfeed->getUrl(),0);

     /*  char fileo[1000],file[1000]; 
       sprintf(fileo,"%sdump%d-old.txt",BBpluginExport_GetPlugin()->directory,f);
       sprintf(file,"%sdump%d.txt",BBpluginExport_GetPlugin()->directory,f);
       unlink(fileo);
       rename(file,fileo);
       FILE * fp=fopen(file,"w+b");
       if (fp==NULL) MessageBox(0,"err",0,0);
       long s;
       void *d=response.getData(&s);
       fwrite(d,s,1,fp);
       fclose(fp);*/

       const char *data;
       long size;
       data=(const char *)response.getContentData(&size);
       if (size==0) {
          rssfeed->setError(" - ERROR: Document contains no data");
          updateList();
          continue;
       }
       TiXmlDocument doc;
       // neda se spolehnout na vracenou hodnotu, dela si to co chce
       doc.Parse(data);
       // error parsing xml
       if (doc.Error()) {
          rssfeed->setError(" - ERROR: Error parsing XML file");
          updateList();
          continue;
       }


   //MessageBox(0,doc.LoadFile()?"loaded":doc.ErrorDesc(),0,0);

       //TiXmlDeclaration *d=doc.ToDeclaration();
       //MessageBox(0,d->Encoding(),0,0);

       TiXmlHandle docHandle(&doc);
       TiXmlElement *itemsource, *infosource, *item;

       bool utf8=false;
       TiXmlNode *declnode=docHandle.FirstChild().ToNode();
       if (declnode) {
          TiXmlDeclaration *decl=declnode->ToDeclaration();
          utf8=decl && !stricmp(decl->Encoding(),"utf-8");
       }

       // RSS feeds
       if (docHandle.FirstChild("rss").ToElement()) {

          itemsource=docHandle.FirstChild("rss").FirstChild("channel").ToElement(); // RSS2.0
          infosource=itemsource;
          if (!itemsource) {
             itemsource=docHandle.FirstChild("rdf:RDF").ToElement(); // RSS1.0
             infosource=docHandle.FirstChild("rdf:RDF").FirstChild("channel").ToElement();
             if (!itemsource || !infosource) {
                rssfeed->setError(" - ERROR: Invalid document format");
                updateList();
                continue;
             }
          }

          const char *text;
          char *utf8text;
          text=getElementText(infosource->FirstChildElement("title"));
          if (utf8) {
             utf8text=RSSReader::UTF8ToAnsi(text);
             rssfeed->setTitle(utf8text);
             if (utf8text) delete utf8text;
          } else rssfeed->setTitle(text);
          rssfeed->setLink(getElementText(infosource->FirstChildElement("link")));
          text=getElementText(infosource->FirstChildElement("description"));
          if (utf8) {
             utf8text=RSSReader::UTF8ToAnsi(text);
             rssfeed->setDescription(utf8text);
             if (utf8text) delete utf8text;
          } else rssfeed->setDescription(text);

          item=itemsource->FirstChildElement("item");
          while (item) {
                RSSItem *rssitem=new RSSItem();
                text=getElementText(item->FirstChildElement("title"));
                if (utf8) {
                   utf8text=RSSReader::UTF8ToAnsi(text);
                   rssitem->setTitle(utf8text);
                   if (utf8text) delete utf8text;
                } else rssitem->setTitle(text);
                text=getElementText(item->FirstChildElement("description"));
                if (utf8) {
                   utf8text=RSSReader::UTF8ToAnsi(text);
                   rssitem->setDescription(utf8text);
                   if (utf8text) delete utf8text;
                } else rssitem->setDescription(text);
                rssitem->setLink(getElementText(item->FirstChildElement("link")));
                //MessageBox(0,rssitem->getDescription(),rssitem->getTitle(),0);

                rssfeed->addItem(rssitem);
                item=item->NextSiblingElement();
          } 
       }
       else

       // Atom feeds
       if (docHandle.FirstChild("feed").ToElement()) {

          infosource=itemsource=docHandle.FirstChild("feed").ToElement();

          const char *text;
          char *utf8text;
          text=getElementText(infosource->FirstChildElement("title"));
          if (utf8) {
             utf8text=RSSReader::UTF8ToAnsi(text);
             rssfeed->setTitle(utf8text);
             if (utf8text) delete utf8text;
          } else rssfeed->setTitle(text);
          rssfeed->setLink(getElementAttribute(infosource->FirstChildElement("link"),"href"));
          text=getElementText(infosource->FirstChildElement("subtitle"));
          if (utf8) {
             utf8text=RSSReader::UTF8ToAnsi(text);
             rssfeed->setDescription(utf8text);
             if (utf8text) delete utf8text;
          } else rssfeed->setDescription(text);

          item=itemsource->FirstChildElement("entry");
          while (item) {
                RSSItem *rssitem=new RSSItem();
                text=getElementText(item->FirstChildElement("title"));
                if (utf8) {
                   utf8text=RSSReader::UTF8ToAnsi(text);
                   rssitem->setTitle(utf8text);
                   if (utf8text) delete utf8text;
                } else rssitem->setTitle(text);
                text=getElementText(item->FirstChildElement("summary"));
                if (utf8) {
                   utf8text=RSSReader::UTF8ToAnsi(text);
                   rssitem->setDescription(utf8text);
                   if (utf8text) delete utf8text;
                } else rssitem->setDescription(text);
                rssitem->setLink(getElementAttribute(item->FirstChildElement("link"),"href"));
                //MessageBox(0,rssitem->getDescription(),rssitem->getTitle(),0);

                rssfeed->addItem(rssitem);
                item=item->NextSiblingElement();
          } 
       }

       updateList();

       // aby to pri prvnim nacteni nepipalo sto let
       if (plugindata.popupNotify)
          if (oldfeed.getNumItems()!=0)	
             for (int n=0;n<rssfeed->getNumItems();n++) 
                 if (!oldfeed.getItemByUrl(rssfeed->getItem(n)->getLink())) {
                    char broam[1024];
                    sprintf(broam,"@BBpopup.addevent %lu \"%s: %s\"",
                       plugin->hwnd,
                       rssfeed->getCaption(),
                       rssfeed->getItem(n)->getTitle());
                    BBpluginExport_SendBroam(broam);
                 }
	   //MessageBox(0,"4",0,0);
   }

   thread_running=false;
   return(0);
}

const char * getElementText (TiXmlElement * elem)
{
   return(elem?elem->GetText():NULL);
}

const char * getElementAttribute (TiXmlElement * elem, const char * attribute)
{
   return(elem?elem->Attribute(attribute):NULL);
}
