
#ifndef RSSREADER_H__
#define RSSREADER_H__

#include <string>
#include <vector>
#include <windows.h>

class RSSItem
{
   private:
      std::string title, link, description;
   public:
      RSSItem ();
      RSSItem (RSSItem *);
      const char * getTitle (void)         { return(title.c_str()); }
      const char * getLink (void)          { return(link.c_str()); }
      const char * getDescription (void)   { return(description.c_str()); }
      void setTitle (const char * t)       { title=t==NULL?"":t; }
      void setLink (const char * l)        { link=l; }
      void setDescription (const char * d) { description=d==NULL?"":d; }
};

class RSSFeed
{
   private:
      std::string caption, url;
      std::string title, link, description;
      std::vector <RSSItem *> items;
      bool enabled;
      int maxItems;
   public:
      RSSFeed (const char *, const char *, bool);
      RSSFeed (RSSFeed *);
      ~RSSFeed ();
      void clear (void);
      const char * getCaption (void)      { return(caption.c_str()); }
      const char * getUrl (void)          { return(url.c_str()); }
      void addItem (RSSItem *);
      int getNumItems (void);
      RSSItem * getItem (int);
      RSSItem * getItemByUrl (const char *);
      const char * getTitle (void)        { return(title.c_str()); }
      const char * getLink (void)         { return(link.c_str()); }
      const char * getDescription (void)  { return(description.c_str()); }
      bool isEnabled (void)               { return(enabled); }
      void setCaption (const char *);
      void setUrl (const char *);
      void setTitle (const char *);
      void setError (const char *);
      void setLink (const char *);
      void setDescription (const char *);
      void toggleEnabled (void)           { enabled=!enabled; }
      int getMaxItems (void)              { return(maxItems); }
      void setMaxItems (int m)            { maxItems=m; }
};

class RSSReader
{
   private:
      std::vector <RSSFeed *> feeds;
   public:
      RSSReader ();
      ~RSSReader ();
      void clear (void);             
      void addFeed (RSSFeed *);
      int getNumFeeds (void)              { return(feeds.size()); }
      RSSFeed * getFeedByUrl (const char *);
      RSSFeed * getFeed (int);

      static char * UTF8ToAnsi (const char *);
};

#endif