
#ifndef WEATHER_H__
#define WEATHER_H__

#include <vector>
#include "..\BBpluginSDK\Labels.h"
#include <gdiplus.h>

#define  WEATHER_MAX  1024
#define  ID_MAX       128

class Icon 
{
   private:
      char              id[ID_MAX];
      Gdiplus::Bitmap * icon;
   public:
      Icon (const char * t, Gdiplus::Bitmap * i) { 
         icon=i; 
         strncpy(id,t,ID_MAX); 
      }
      ~Icon (void)                         { delete icon; }
      Gdiplus::Bitmap * getHandle (void)   { return(icon); }
      const char * getID (void)            { return(id); }
};

class IconList
{
   private:
      std::vector <Icon *> items;
   public:
      ~IconList (void)                    { clear(); }
      void clear (void)                   { for (int n=0;n<items.size();n++) delete items[n]; items.clear(); }
      void addIcon (Icon * i)             { items.insert(items.end(),i); }
      int getNumIcons (void)              { return(items.size()); }
      Icon * getIcon (int n)              { return(items[n]); }
      Icon * getIconByID (const char * id) {
         if (id==NULL || strlen(id)==0) return(NULL);    // prazdny retezec neni ID
         for (int n=0;n<items.size();n++) 
             if (!strcmp(items[n]->getID(),id)) return(items[n]);
         return(NULL);
      }
};


class Location 
{
   private:
      char   name[WEATHER_MAX],url[WEATHER_MAX],deffile[WEATHER_MAX],
             sky[ID_MAX];
   public:
      int    x,y;
      LabelList * labels;

      Location (int xx, int yy, const char * n, const char * u, const char * d) {
         x=xx;
         y=yy;
         strncpy(name,n,WEATHER_MAX);
         strncpy(url,u,WEATHER_MAX);
         strncpy(deffile,d,WEATHER_MAX);
         memset(sky,0,ID_MAX);
      }
      ~Location (void)                    { labels->clear(); }
      const char * getName (void)         { return(name); }
      const char * getUrl (void)          { return(url); }
      const char * getDefFile (void)      { return(deffile); }
      const char * getSky (void)          { return(sky); }
      void setSky (const char * s)        { strcpy(sky,s); }
};

class LocationList
{
   private:
      std::vector <Location *> items;
   public:
      ~LocationList (void)                { clear(); }
      void clear (void)                   { for (int n=0;n<items.size();n++) delete items[n]; items.clear(); }
      void addLocation (Location * i)     { items.insert(items.end(),i); }
      int getNumLocations (void)          { return(items.size()); }
      Location * getLocation (int n)      { return(items[n]); }
};

class Token
{
   private:
      char  id[WEATHER_MAX],regexp[WEATHER_MAX];
   public:
      Token (const char * i, const char * r) {
         strncpy(id,i,WEATHER_MAX);
         strncpy(regexp,r,WEATHER_MAX);
      }
      const char * getID (void)           { return(id); }
      const char * getRegexp (void)       { return(regexp); }
};

class TokenList
{
   private:
      std::vector <Token *> items;
   public:
      ~TokenList (void)                   { clear(); }
      void clear (void)                   { for (int n=0;n<items.size();n++) delete items[n]; items.clear(); }
      void addToken (Token * i)           { items.insert(items.end(),i); }
      int getNumTokens (void)             { return(items.size()); }
      Token * getToken (int n)            { return(items[n]); }
      Token * getTokenByID (const char * id) {
         if (id==NULL || strlen(id)==0) return(NULL);    // prazdny retezec neni ID
         for (int n=0;n<items.size();n++) 
             if (!strcmp(items[n]->getID(),id)) return(items[n]);
         return(NULL);
      }
};

class Weather
{
   private:
      char  iconID[WEATHER_MAX],keyword[WEATHER_MAX];
   public:
      Weather (const char * i, const char * k) {
         strncpy(iconID,i,WEATHER_MAX);
         strncpy(keyword,k,WEATHER_MAX);
      }
      const char * getIconID (void)       { return(iconID); }
      const char * getKeyword (void)      { return(keyword); }
};

class WeatherList
{
   private:
      std::vector <Weather *> items;
   public:
      ~WeatherList (void)                 { clear(); }
      void clear (void)                   { for (int n=0;n<items.size();n++) delete items[n]; items.clear(); }
      void addWeather (Weather * i)       { items.insert(items.end(),i); }
      int getNumWeathers (void)           { return(items.size()); }
      Weather * getWeather (int n)        { return(items[n]); }
};

#endif