
#include "BBsystraffic.h"

PLUGIN_STRUCT *plugin;

struct plugindata {
   struct TIMERS {
      UINT cpu,mem,bat,uptime,ip,disk;
   } timers;
   bool reconfUpdate;
   DATATYPE fullQuota;
} plugindata;

struct ICONS {
#ifdef GDIPLUS_IN_USE
   Gdiplus::Bitmap *freedisk,*fulldisk,*battery,*acon,*charge;
#else
   HICON freedisk,fulldisk,battery,acon,charge;
#endif
   
} icons;
DATATYPE freespace[MAX_DRIVES];
bool charging,ACon;
struct TIMERSID {
   UINT cpu,mem,bat,uptime,ip,disk;
} timersID;

Graph *graph[2];
LabelList labellist,disks;


const char * BBpluginCallback_InfoQuery (int field)
{
	switch (field) {
		case PLUGIN_NAME:      return(szAppName);
      case PLUGIN_CLASSNAME: return(szClassName);
		case PLUGIN_VERSION:   return(szInfoVersion);  
		case PLUGIN_AUTHOR:    return(szInfoAuthor);   
		case PLUGIN_RELEASE:   return(szInfoRelDate); 
		case PLUGIN_LINK:      return(szInfoLink);   
		case PLUGIN_EMAIL:     return(szInfoEmail);   
      case PLUGIN_NAMEVER:   return(szVersion);
	}
   return(NULL);
}

bool BBpluginCallback_OnLoadPlugin()
{ 
   plugin=BBpluginExport_GetPlugin();

   if (!InitSystemMonitor()) {
      MessageBox(0,"Error initializing system monitor!","Fatal Error",MB_OK);
      return(false);
   }

   return(true);
}

void BBpluginCallback_OnUnloadPlugin()
{
   KillTimer(NULL,timersID.cpu);
   KillTimer(NULL,timersID.mem);
   KillTimer(NULL,timersID.bat);
   KillTimer(NULL,timersID.uptime);
   KillTimer(NULL,timersID.ip);
   KillTimer(NULL,timersID.disk);
   labellist.clear();
   disks.clear();
   delete graph[GCPU];
   delete graph[GMEM];
#ifdef GDIPLUS_IN_USE
   delete icons.freedisk;
   delete icons.fulldisk;
   delete icons.battery;
   delete icons.acon;
   delete icons.charge;
#else
   if (icons.freedisk) DestroyIcon(icons.freedisk);
   if (icons.fulldisk) DestroyIcon(icons.fulldisk);
   if (icons.battery) DestroyIcon(icons.battery);
   if (icons.acon) DestroyIcon(icons.acon);
   if (icons.charge) DestroyIcon(icons.charge);
#endif
   DestroySystemMonitor();
}

bool BBpluginCallback_OnMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   return(false);
}

void BBpluginCallback_OnBroam (const char * broam)
{
}

#ifdef GDIPLUS_IN_USE
void BBpluginCallback_OnPaint (HDC hdc)
{
   Gdiplus::Graphics g(hdc);

   Gdiplus::Font font(hdc,BBpluginExport_GetBB()->font);
   COLORREF c=plugin->windowStyleItem?plugin->windowStyleItem->TextColor:RGB(0,0,0);
   Gdiplus::SolidBrush brush(Gdiplus::Color(255,GetRValue(c),GetGValue(c),GetBValue(c)));

   for (int gr=GCPU;gr<=GMEM;gr++) graph[gr]->paint(g,hdc,font);

   labellist.paintLabels(g,font,brush);

   for (int d=0;d<disks.getNumLabels();d++) {
       Label *label=disks.getLabel(d);
       label->paint(g,font,brush);
       Gdiplus::Bitmap * icon=freespace[d]>plugindata.fullQuota?icons.freedisk:icons.fulldisk;
       if (icon!=NULL)
          g.DrawImage(icon,label->iconX,label->iconY,icon->GetWidth(),icon->GetHeight());
   }

   Gdiplus::Bitmap * icon=(ACon?(charging?icons.charge:icons.acon):icons.battery);
   if (icon!=NULL)
      g.DrawImage(icon,labellist.getLabel(LBAT)->iconX,labellist.getLabel(LBAT)->iconY,icon->GetWidth(),icon->GetHeight());
}
#else
void BBpluginCallback_OnPaint (HDC hdc)
{
   for (int g=GCPU;g<=GMEM;g++) graph[g]->paint(hdc);

   labellist.paintLabels(hdc);

   for (int d=0;d<disks.getNumLabels();d++) {
       Label *label=disks.getLabel(d);
       label->paint(hdc);
       HICON icon=freespace[d]>plugindata.fullQuota?icons.freedisk:icons.fulldisk;
       if (icon!=NULL)
          DrawState(hdc,0,0,(long)icon,0,label->iconX,label->iconY,0,0,DST_ICON);
   }

   DrawState(hdc,0,0,
             (long)(ACon?(charging?icons.charge:icons.acon):icons.battery),
             0,labellist.getLabel(LBAT)->iconX,labellist.getLabel(LBAT)->iconY,0,0,DST_ICON);
}
#endif

bool BBpluginCallback_OnCreateMenu (Menu * menu)
{
   return(false);
}

void BBpluginCallback_OnConfigure (void)
{
   disks.clear();
   labellist.clear();

   plugindata.reconfUpdate=BBplugin_ReadBool("reconfigureUpdate",false);

   for (int g=GCPU;g<=GMEM;g++) {
       if (graph[g]) delete graph[g];
       graph[g]=new Graph(g==GCPU?"cpuGraph":"memoryGraph");
       graph[g]->setFixedMaxValue(100);
   }

   labellist.addLabel(new Label("ip"));
   labellist.addLabel(new Label("totalMemory"));
   labellist.addLabel(new Label("usedMemory"));
   labellist.addLabel(new Label("freeMemory"));
   labellist.addLabel(new Label("battery"));
   labellist.addLabel(new Label("systemUptime"));

   plugindata.timers.cpu=BBplugin_ReadInt("timers.cpu",1000);
   plugindata.timers.mem=BBplugin_ReadInt("timers.memory",1000);
   plugindata.timers.bat=BBplugin_ReadInt("timers.battery",2000);
   plugindata.timers.uptime=BBplugin_ReadInt("timers.uptime",60000);
   plugindata.timers.ip=BBplugin_ReadInt("timers.ip",600000);
   plugindata.timers.disk=BBplugin_ReadInt("timers.disk",5000);
   plugindata.fullQuota=BBplugin_ReadInt("fullQuota",500)*1000000L;

#ifdef GDIPLUS_IN_USE
   if (icons.freedisk) delete icons.freedisk;
   icons.freedisk=BBplugin_LoadImage(BBplugin_ReadString("icons.freedisk",NULL));
   if (icons.fulldisk) delete icons.fulldisk;
   icons.fulldisk=BBplugin_LoadImage(BBplugin_ReadString("icons.fulldisk",NULL));
   if (icons.battery) delete icons.battery;
   icons.battery=BBplugin_LoadImage(BBplugin_ReadString("icons.battery",NULL));
   if (icons.acon) delete icons.acon;
   icons.acon=BBplugin_LoadImage(BBplugin_ReadString("icons.ACon",NULL));
   if (icons.charge) delete icons.charge;
   icons.charge=BBplugin_LoadImage(BBplugin_ReadString("icons.batteryCharge",NULL));
#else
   if (icons.freedisk) DestroyIcon(icons.freedisk);
   icons.freedisk=BBplugin_LoadIcon(BBplugin_ReadString("icons.freedisk",NULL));
   if (icons.fulldisk) DestroyIcon(icons.fulldisk);
   icons.fulldisk=BBplugin_LoadIcon(BBplugin_ReadString("icons.fulldisk",NULL));
   if (icons.battery) DestroyIcon(icons.battery);
   icons.battery=BBplugin_LoadIcon(BBplugin_ReadString("icons.battery",NULL));
   if (icons.acon) DestroyIcon(icons.acon);
   icons.acon=BBplugin_LoadIcon(BBplugin_ReadString("icons.ACon",NULL));
   if (icons.charge) DestroyIcon(icons.charge);
   icons.charge=BBplugin_LoadIcon(BBplugin_ReadString("icons.batteryCharge",NULL));
#endif

   int dsks=BBplugin_ReadInt("diskCount",0);
   for (int i=0;i<dsks;i++) {
       char txt[1024];
       sprintf(txt,"disk[%d]",i+1);
       disks.addLabel(new Label(txt));
   }

   ResetTimers();

   if (plugindata.reconfUpdate) UpdateAll();
}

void BBpluginCallback_OnSaveConfiguration (void)
{
}


void UpdateAll (void)
{
   TimerProcCpu();
   TimerProcMemory();
   TimerProcBattery();
   TimerProcUptime();
   TimerProcIP();
   TimerProcDisk();
}

void ResetTimers (void)
{
   if (timersID.cpu) KillTimer(NULL,timersID.cpu);
   timersID.cpu=SetTimer(NULL,NULL,plugindata.timers.cpu,(TIMERPROC)TimerProcCpu);
   if (timersID.mem) KillTimer(NULL,timersID.mem);
   timersID.mem=SetTimer(NULL,NULL,plugindata.timers.mem,(TIMERPROC)TimerProcMemory);
   if (timersID.bat) KillTimer(NULL,timersID.bat);
   timersID.bat=SetTimer(NULL,NULL,plugindata.timers.bat,(TIMERPROC)TimerProcBattery);
   if (timersID.uptime) KillTimer(NULL,timersID.uptime);
   timersID.uptime=SetTimer(NULL,NULL,plugindata.timers.uptime,(TIMERPROC)TimerProcUptime);
   if (timersID.ip) KillTimer(NULL,timersID.ip);
   timersID.ip=SetTimer(NULL,NULL,plugindata.timers.ip,(TIMERPROC)TimerProcIP);
   if (timersID.disk) KillTimer(NULL,timersID.disk);
   timersID.disk=SetTimer(NULL,NULL,plugindata.timers.disk,(TIMERPROC)TimerProcDisk);
}

TIMERPROC TimerProcDisk (void)
{
   for (int d=0;d<disks.getNumLabels();d++) {
       Label *label=disks.getLabel(d);
       freespace[d]=GetDiskFreeSpace(label->getID());
       valueOut(label->getContentPtr(),freespace[d]);
   }

   BBpluginExport_Repaint();
   return(0);
}
TIMERPROC TimerProcIP (void)
{
   GetLocalIP(labellist.getLabel(LIP)->getContentPtr());

   BBpluginExport_Repaint();
   return(0);
}
TIMERPROC TimerProcCpu (void)
{
   int cpu=GetCPULoad();
   graph[GCPU]->addValue(cpu);
   sprintf(graph[GCPU]->getDisplayTextPtr(),"CPU %d%%",cpu);

   BBpluginExport_Repaint();
   return(0);
}
TIMERPROC TimerProcMemory (void)
{
   DWORD memTotal,memAvail,memLoad;
   GetMemoryStatus(&memTotal,&memAvail,&memLoad);
   graph[GMEM]->addValue(memLoad);
   sprintf(graph[GMEM]->getDisplayTextPtr(),"MEM %d%%",memLoad);

   valueOut(labellist.getLabel(LMT)->getContentPtr(),memTotal);
   valueOut(labellist.getLabel(LMU)->getContentPtr(),memTotal-memAvail);
   valueOut(labellist.getLabel(LMF)->getContentPtr(),memAvail);

   BBpluginExport_Repaint();
   return(0);
}
TIMERPROC TimerProcBattery (void)
{
   bool hasBattery;
   int percentage;
   GetPowerStatus(&hasBattery,&ACon,&charging,&percentage);
   if (hasBattery) sprintf(labellist.getLabel(LBAT)->getContentPtr(),"%d%%",percentage);
      else strcpy(labellist.getLabel(LBAT)->getContentPtr(),"AC");

   BBpluginExport_Repaint();
   return(0);
}
TIMERPROC TimerProcUptime (void)
{
   int hours,minutes;
   GetSystemTime(&hours,&minutes);
   sprintf(labellist.getLabel(LUT)->getContentPtr(),"%d:%02d",hours,minutes);

   BBpluginExport_Repaint();
   return(0);
}

void valueOut (char * text, DATATYPE value)
{
   if (value<_2NA10) sprintf(text,"%I64d B",value);
   if (value>=_2NA10 && value<_2NA20) sprintf(text,"%.3f KB",(double)value/_2NA10);
   if (value>=_2NA20 && value<_2NA30) sprintf(text,"%.1f MB",(double)value/_2NA20);
   if (value>=_2NA30) sprintf(text,"%.2f GB",(double)value/_2NA30);
}

