
#include "SystemMonitor.h"

NTQUERYSYSTEMINFORMATION NtQuerySystemInformation;
HMODULE ntdllmodule=NULL;

const int SystemBasicInformation=0,
          SystemPerformanceInformation=2,
          SystemTimeInformation=3;

long double last_system_time=0,last_idle_time=0;
int number_processors;
bool winnt;

bool InitSystemMonitor (void)
{
	OSVERSIONINFO osinfo;
	ZeroMemory(&osinfo,sizeof(OSVERSIONINFO));
	osinfo.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
	GetVersionEx(&osinfo);
	winnt=osinfo.dwPlatformId==VER_PLATFORM_WIN32_NT;

   if (winnt) {
	   if ((ntdllmodule=LoadLibrary("ntdll.dll"))==NULL) return(false);

      NtQuerySystemInformation=(NTQUERYSYSTEMINFORMATION)GetProcAddress(ntdllmodule,"NtQuerySystemInformation");
      if (NtQuerySystemInformation==NULL) {
         DestroySystemMonitor();
         return(false);
      }

      BOOL status;
      SYSTEM_BASIC_INFORMATION basicinformation;
      status=NtQuerySystemInformation(SystemBasicInformation,&basicinformation,sizeof(basicinformation),NULL);
      if (status!=NO_ERROR) {
         DestroySystemMonitor();
         return(false);
      }

      //Record the number of processors
      number_processors=basicinformation.bKeNumberProcessors;

      if (number_processors < 1 || number_processors > 64) {
         DestroySystemMonitor();
         return(false);
      }
   }

   WSADATA wsa;
   if (WSAStartup(MAKEWORD(2,0),&wsa)!=0) return(false);

   return(true);
}

void DestroySystemMonitor (void)
{
   if (ntdllmodule) FreeLibrary(ntdllmodule);
   WSACleanup();
}

int GetCPULoad()
{
   double foundvalue;

   if (winnt) {
      long double diff_system_time;
      long double diff_idle_time;
      SYSTEM_TIME_INFORMATION SysTimeInfo;
      SYSTEM_PERFORMANCE_INFORMATION SysPerfInfo;

      BOOL status=NtQuerySystemInformation(SystemTimeInformation,&SysTimeInfo,sizeof(SysTimeInfo),NULL);
      if (status==NO_ERROR) status=NtQuerySystemInformation(SystemPerformanceInformation,&SysPerfInfo,sizeof(SysPerfInfo),NULL);

      if (status==NO_ERROR) {
         long double current_system_time = (long double) SysTimeInfo.liKeSystemTime.QuadPart;
         long double current_idle_time = (long double) SysPerfInfo.liIdleTime.QuadPart;

         if (last_system_time == 0) foundvalue=0.0;
         else {
            diff_system_time=current_system_time-last_system_time;
            diff_idle_time=current_idle_time-last_idle_time;

            foundvalue=1.0-(((diff_idle_time/diff_system_time))/number_processors);
            if (foundvalue<0.0) foundvalue=0.0;
            if (foundvalue>1.0) foundvalue=1.0;
         }

         last_system_time=current_system_time;
         last_idle_time=current_idle_time;
      } 

      return((int)(foundvalue*100));
   }
   // WIN9x
   else {
      HKEY hkey;
      DWORD dwDataSize;
      DWORD dwType;
      DWORD dwCpuUsage;
			
      RegOpenKeyEx(HKEY_DYN_DATA,"PerfStats\\StatData",0,KEY_QUERY_VALUE,&hkey);
      dwDataSize=sizeof(dwCpuUsage);
      RegQueryValueEx(hkey,"KERNEL\\CPUUsage",NULL,&dwType,(LPBYTE)&dwCpuUsage,&dwDataSize);
      RegCloseKey(hkey);

		return((int)(((double)dwCpuUsage)/100));
   }
}

__int64 GetDiskFreeSpace (const char * path)
{
   __int64 freeBytes,bytes,freeBytesPerDisk;
   GetDiskFreeSpaceEx(path,(PULARGE_INTEGER)&freeBytes,(PULARGE_INTEGER)&bytes,(PULARGE_INTEGER)&freeBytesPerDisk);
   return(freeBytesPerDisk);
}

void GetLocalIP (char * buffer)
{
   gethostname(buffer,100);
   hostent *h=gethostbyname(buffer);
   strcpy(buffer,inet_ntoa(*(in_addr*)h->h_addr_list[0]));
}

void GetMemoryStatus (DWORD * total, DWORD * avail, DWORD *load) 
{
   MEMORYSTATUS ms;
   GlobalMemoryStatus(&ms);
   *total=ms.dwTotalPhys;
   *avail=ms.dwAvailPhys;
   *load=ms.dwMemoryLoad;
}

void GetPowerStatus (bool * hasBattery, bool * ACon, bool * charging, int * percentage)
{
   SYSTEM_POWER_STATUS sps;
   GetSystemPowerStatus(&sps);
   *hasBattery=!((sps.BatteryFlag&128)==128);
   *ACon=sps.ACLineStatus==1;
   *charging=(sps.BatteryFlag&8)==8;
   *percentage=sps.BatteryLifePercent;
}

void GetSystemTime (int * hours, int * minutes)
{
   *minutes=GetTickCount()/1000/60,
   *hours=*minutes/60;
   *minutes-=*hours*60;
}